;======================== header ===============================================
;+
; NAME:
;   cactus_arch 
; PURPOSE:
;   truncate all files in pipe to a limited size and store the deleted parts in the 
;   corresponding archive directory
; CATEGORY:
;   cactus 
; INPUTS:
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; MODIFICATION HISTORY:
;   2002/06/03, ver 1.00, David Berghmans created  
;   2003/06/10, DB, pipe dir removed
;   2004/02/19, ER, do not limit master.log file
;   2005/09/05, ER, change spawn, 'ls '+logdir,ls to spawn, 'ls '+logdir+'*.log',ls to not have directory Oldlog included
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;-
;======================== end header ===========================================
PRO cactus_arch, ndays=ndays, no_logfile=no_logfile 

IF NOT KEYWORD_SET(ndays) THEN ndays=4

   cactus_preamble, dir,myname  
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------
 ; program wide declarations
 
 ; settings
   maxndays = ndays
     ; MAXimum Number of DAYS that we keep in the pipeline,
     ; older data is deleted


 ; loop over telescopes 
   instr=['c2','c3']
   for inr = 0, n_elements(instr)-1 do begin
    
    ; log
      printf,log, '   Processing '+instr[inr]

    ; check if files exist
      indexfile      =  dir.work+instr[inr]+'index.sav'
      ar_images_file =  dir.work+instr[inr]+'_ar_images.sav'
      xy_images_file =  dir.work+instr[inr]+'_xy_images.sav'
      files_exist =(file_test(indexfile) and $
                    file_test(ar_images_file) and $
                    file_test(xy_images_file))
      IF not files_exist THEN BEGIN
         printf, log, 'One of the following files is missing: '
	 printf, log, indexfile
	 printf, log, ar_images_file
	 printf, log, xy_images_file
	 printf, log, ' ... skipping.'
      ENDIF ELSE BEGIN            		
       ; get input
         restore, indexfile
         nimages = n_elements(index)
  
       ; check obstime
         latestday   = FIX(index[nimages-1].obstime); largest integer < index[nimages-1].obstime
         fewdaysago  = latestday-maxndays   
          
       ; keep only recent data
         recent      = where(FIX(index.obstime) GE fewdaysago,nrecent)
         firstrecent = min(recent)
       ; chop and save if necessary
         IF firstrecent NE 0 THEN BEGIN
            index       = index[firstrecent:*]
            save, filename=indexfile, index
	    ;
            restore, ar_images_file
	    polar       = polar[*,*,firstrecent:*]  
            save, filename=ar_images_file, polar
	    ;
            restore, xy_images_file
	    xycube      = xycube[*,*,firstrecent:*]  
            save, filename=xy_images_file, xycube
	    ;
	    
          ; log
            printf, log, '      Number of images removed: ', nimages-nrecent
         ENDIF ELSE printf, log, '      No change'
       ENDELSE ; files_exist
   
   endfor ; inr = 0, n_elements(instr)

 ; ------------------------closing remarks (Non generic!)-----------------
   
; ------------------------closing remarks-------------------------------
   printf, log, '   Chopping log files to last 1000 files will be done'
   printf, log, '   after closing of this log file.'
  
   cactus_loginfo, myname, log, no_logfile=no_logfile, /close
 ; ----------------------------------------------------------------------


 ; limit excessive log files 
  spawn, 'ls '+dir.log+'*.log',ls
  FOR ithlogfile= 0, n_elements(ls)-1 do begin
    if ls[ithlogfile] ne 'CACTUS_MASTER.log' then begin
     spawn, 'tail -n 1000 '+ls[ithlogfile] + ' > '+dir.log+'temp'
     spawn, 'mv '+dir.log+'temp '+ls[ithlogfile]
     endif
  ENDFOR   
END
