;======================== header ===============================================
;+
; NAME:
;   cactus_catalo
; PURPOSE:
;   find clusters in the v,\theta,t cubes and write catalogues
; CATEGORY:
;   cactus 
; INPUTS:
;
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;   set width=min_w to define the minimal angular width of the CMEs, default is 8 degrees
; OUTPUTS:
;   flags/out
; COMMON BLOCKS:
;   none
;
; CALLS:
;   cactus_translate.pro
;   cactus_starttime.pro
;   cactus_mailer.pro
;   cactus_infoline.pro
; TO DO:
;
;  -this program is a mess thrown together from 2 previous programs which
;    were a mess themselves.
;  - should collect the relevant images and make a movie per CME
;
; MODIFICATION HISTORY:
;   2002/05/22, ver 1.00, David Berghmans
;               created on the basis of spwp/make_cme_cat3.pro and
;                                       spwp/find_cme.pro
;   2002/06/02, ver 1.00, DB, cleaned up, (partially ?) debugged
;   2003/01/16, ver 1.01, DB, cmemap that have started before beginning of
;                             sequence are now excluded
;   2003/01/17. ver 1.1. DB, now doing C3 and C2
;   2003/03/27, test: we are just using the 13 lowest velocity bins
;   2003/04/08, now analysing tvanglecube in 3D
;   2003/04/25, DB, now working on combined c2c3 data
;   2003/05/17  KJ, calculates now 'da' and 'pa' correctly. It doesn't matter of nangles = 360 or not.         
;   2003/06/10, DB, pipe dir removed
;   2003/12/16, ER, starting time corrected + connection of CME over occulter
;   2004/01/08, ER, making threshold for tvanglecube cadence-dependent
;   2004/01/09, ER, adding line for also calculating velocities of latest CMEs
;   2004/04/01, DB, catalo now sends email alerts via cactusmailer
;   2004/04,02, ER, added extra calculation for startingtimes CMEs
;   2004/04,04, ER, now calculates median velocity instead of average v + boxplot
;   2004/11/25, ER, multiple entries possible now in cactusmailer, cactusmailer only started when needed
;   2004/12/08, ER, Bug corrected, firstobstime=obstime[start_index[i]]
;   2004/12/17, ER, velplot moved to out.pro
;   2005/02/03, ER, cactusmailer not activated if CME is 'marginal case   ' 
;   2005/02/03, DB, cactusmailer activiated only if (obstime_now-firstobstime LE 3)
;   2006/08/01, ER, Remove all pylon detections (detection is defined as false if width < 40 deg and pa within 15 deg of the pylon)
;   2006/08/07, ER, added (sigma ne 0) to condition for sending halo alert (sigma = 0 is signature of false detection)
;   2006/08/11, ER, undo threshold-cadence dependence
;  ------------------ version 2.0.0 ----------------------------
;  ------------------ version 2.0.1 ----------------------------
;   2007/08/06, ER, make cme-structure (incorporate old change from 2006/02/20 into revised versions)
;                   need new/updated routines: infoline.pro (function in lib_cactus)/var.pro (cmecat-stucture added)
;   2007/08/06, ER, give all CMEs a number, sorted on t0, give all flows a number (this is to make the structure work without track.pro)
;   2007/08/07, ER, the header of the cmecat.txt is now made by make_header.pro
;  ------------------ version 2.1.0 ----------------------------
;  ------------------ version 2.2.0 ----------------------------
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2007/12/19, ER, make minimum CME width a keyword. Default is original value.
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/01/09, ER, comment out info into cmecat[0] 
;   2008/03/12, ER, remove crontab keyword, always use Z-buffer for images
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/14, ER, extract 'absolute' cactus parameters from !cactusfov and not from fov.sav
;   2008/03/19, ER, Quit IDL if input files do not exist
;   2008/03/20, ER, get cactus_header info in cactus_header (first and last images used)
;   2008/03/21, ER, do not write cmecat.txt here, do it in cactus_out.pro
;   2008/04/10, ER, cactusindex expanded with obsrvtry/instrument and detector info and also end date/time of event, also sort cmecat before saving
;   2008/04/14, ER, repair bug in sorting (also sort the cmemap)
;   2008/05/21, ER, repair other bug in sorting (also sort first and pada) - probably there exists a better construction by sorting just after cactus_starttime
;   2009/02/13, ER, set default min_w to 5 (same as in cactus_master)
;   2009/10/28, ER, minor changes in da and pa
;   2009/11/09, ER, replace exit statement with GOTO, jump to avoid jumping outside IDL
;   2010/07/14, BB, replaced mod 80 by mod 180, delete cmes that start
;                   before given startdate or after given enddate
;
;-
;======================== end header ===========================================
PRO cactus_catalo, width=min_w, no_logfile=no_logfile , instrument=instrument, thresh=thresh

   cactus_preamble, dir,myname,hostname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------ keyword settings ----------------------------
   IF NOT KEYWORD_SET(min_w) 		THEN min_w = 5    	; set default width threshold (degrees)
   IF NOT KEYWORD_SET(thresh)     	THEN thresh = 0.3 	; set default intensity threshold

; ------------------------main code ------------------------------------

 ; Only realtime computers are allowed to send out alerts by email 
   IF strmid(hostname,0,6) eq 'sol026' THEN BEGIN
     printf,log, '   Mailing allowed? Yes !'
     mailing_allowed=1 
   ENDIF ELSE BEGIN
     printf,log, '   Mailing allowed? No !'
     mailing_allowed=0
   ENDELSE
   mail_line=['      ']
   time_line=[0.0]

    ; input
   IF NOT FILE_TEST(dir.work+'tvanglecube.sav') THEN BEGIN
     printf,log, "  Input files do not exist. Jumping to end of program."
     GOTO, jump
   ENDIF

       restore, dir.work+'tvanglecube.sav'

      ; execute cactus_translate.pro to correct tvanglecube in time   
       s=size(tvanglecube_trans,/type)
       IF s eq 0 THEN BEGIN
        printf, log, '   tvanglecube_trans does not exist ! '        
        tvanglecube = 0
        cactus_translate, no_logfile=no_logfile
        printf,log, '   cactus_translate done'
        restore, dir.work+'tvanglecube.sav'
       ENDIF ELSE printf, log, '   tvanglecube_trans already exists ! ' 
    	
    ; take translated tvanglecube
       ;tvanglecube_old=tvanglecube
       tvanglecube=tvanglecube_trans
       tvanglecube_trans = 0

       restore, dir.work+'velocity_settings.sav' ; cadence
       restore, dir.work+'vectors.sav' ; minDT
       restore, dir.work+'index.sav'	;index
  ;restore the julian stattime and the julian endtime
       restore, dir.work+'julstart_end.sav'


     ; dimensions
       sz=size(tvanglecube)
       nimages   = sz[1] &  nimagesp2 = nimages + 2
       nspeeds   = sz[2]
       nangles   = sz[3]
       obstime=index.obstime
 
       sf = 360/nangles  ; size factor, this factor is used to take into account that we work with
                         ; nangles NE 360

    ; make detectionmap

    ; make threshold cadence-dependent, but minimal 0.3 (for LASCO)
       printf, log, '   threshold = ', thresh 

       tangle=transpose(total(tvanglecube ge thresh,2),[1,0]) GE 1
       tangle2=transpose(total(tvanglecube ,2),[1,0])
       mycube=tangle*0

       regions=label_region(tangle,/ALL_NEIGHBORS)
       histgr = HISTOGRAM(regions,REVERSE_INDICES=r)
       largeregions=where(histgr gt 8/sf,nsmall_regions)
       for ith_small=1,nsmall_regions-1 do mycube[r(r[largeregions[ith_small]]:r[largeregions[ith_small]+1]-1)]=1  ; '0' is background

       ;tangle[0:(4/sf),*]=0 & tangle[(338/sf):*,*]=0
       ;regions=label_region(tangle,/ALL_NEIGHBORS)
       ;histgr = HISTOGRAM(regions,REVERSE_INDICES=r)
       ;largeregions=where(histgr gt 10/sf,nsmall_regions)
       ;for ith_small=1,nsmall_regions-1 do mycube[r(r[largeregions[ith_small]]:r[largeregions[ith_small]+1]-1)]=1  ; '0' is background
       
     ; Make a closing-convolution with kernel s=
	;   0   0   0   0   0        
	;   1   1   1   1   1      
	;   0   0   0   0   0     
       mycube2=fltarr(nangles+10+10,nimages+10+10)     ;add zero rows and columns
       mycube2[10:(nangles+9), 10:(nimages+9)]=mycube
       s=bytarr(1+10/sf,3) & s[*,1]=1
       mycube2=morph_close(mycube2,s)

     ; make the connection through the occulter
       connect=(total(mycube2[346/sf:350/sf,*],1) GT 0) and (total(mycube2[354/sf:358/sf,*],1) GT 0)
  
       for connect_i= 346/sf,358/sf do mycube2[connect_i,*]=mycube2[connect_i,*] OR connect
       
       regions=label_region(mycube2,/ALL_NEIGHBORS)
       histgr = HISTOGRAM(regions,REVERSE_INDICES=r)
       smallregions=where(histgr lt 20/sf,nsmall_regions)
  
       for ith_small=1,nsmall_regions-1 do mycube2[r(r[smallregions[ith_small]]:r[smallregions[ith_small]+1]-1)]=0  ; '0' is background

       mycube=mycube2[10:(nangles+9),10:(nimages+9)]
       detectionmap=mycube
     
     ; give remaining cmemap an individual number
       cmemap=label_region(detectionmap,/ALL_NEIGHBORS)
       ncmemap=max(cmemap)
       edgehit    = cmemap*0
       nonedgehit = cmemap*0
       for i=1,ncmemap  do begin   ; zero is background
	 ; location in angle/time space
	   positions=where(cmemap eq i, npositions)  ;'positions' counts over the whole array
	   IF npositions GT 0 THEN BEGIN
	     totalbrightness = total(tangle2[positions])
	     relbrightness=totalbrightness/(npositions>1)
	     left=min(positions mod 180)
	     IF left le 1 THEN BEGIN
                edgehit[positions]=1
                right = max(positions mod 180)
                IF right GE 178 THEN BEGIN
                   nonedgehit[positions]=1
                ENDIF
             ENDIF ELSE BEGIN 
                nonedgehit[positions]=1
             ENDELSE
	     IF relbrightness LT 0.55 or totalbrightness LT 4 THEN detectionmap[positions]=0
           ENDIF
        endfor

    ; make connenction over the occulter
      detectmap=BYTARR(2*nangles, nimages)
      detectmap[0:nangles-1,*]=nonedgehit
      detectmap[nangles:*,*]=edgehit
  ;save, filename=dir.work+'detectionmapbefore.sav',detectmap,nonedgehit,edgehit
      detectmap=cactus_morph_close(detectmap,s)
  ;save, filename=dir.work+'detectionmapafter.sav',detectmap

      cmes=label_region(detectmap,/ALL_NEIGHBORS)
      ncmemap=max(cmes)

  save, filename=dir.work+'CMEmap.sav',cmemap,cmes
  cactus_starttime,no_logfile=no_logfile , instrument=instrument
  restore, dir.work+'startingtime.sav' ;start_index and comment

  ;good cmes
  good = bytarr(ncmemap+1)
  good[*]=1

  ;find position within index of last image with
  ;index.date_obs le juldays[0]-1
  juldays_start_pos = 0
  notfound = 1
  while notfound and juldays_start_pos lt (size(index))[1] do begin
     current_index_obs = julday(uint(strmid(index[juldays_start_pos].date_obs,5,2)),$
                                uint(strmid(index[juldays_start_pos].date_obs,8,2)),$
                                uint(strmid(index[juldays_start_pos].date_obs,0,4)))    
     if current_index_obs ge juldays[0] then notfound = 0 else juldays_start_pos++
  endwhile

  ;find position within index of last image with
  ;index.date_obs le juldays[0]-1
  juldays_end_pos = juldays_start_pos
  notfound = 1
  while notfound and juldays_end_pos lt (size(index))[1] do begin
     current_index_obs = julday(uint(strmid(index[juldays_end_pos].date_obs,5,2)),$
                                uint(strmid(index[juldays_end_pos].date_obs,8,2)),$
                                uint(strmid(index[juldays_end_pos].date_obs,0,4)))    
     if current_index_obs gt juldays[1] then begin
        notfound = 0 
     endif else juldays_end_pos++
  endwhile
  juldays_end_pos--
  
    ; Remove cmes with angle le 8 degrees and pylon detections   
      for icme=1,ncmemap do begin
    	pos=where(cmes eq icme,npos)
    	time=pos/(nangles*2)   	;time
    	angle=(pos-time*nangles*2) 	;angle
    	left=min(angle)  & right=max(angle)
        pa=(((left+right)/2.) mod nangles)*sf
	paReal=(pa+!cactusfov.edge-90) MOD 360		;!cactusfov.edge is measuerd from mathematical theta=0 (=solar west)
	daReal=(right-left)*sf
	; remove small detections
     if (daReal le min_w) then begin
        cmes[pos]=0
        good[icme]=0
     endif
    	; Remove all pylon detections (detection is defined as false if width < 40 deg and pa within 15 deg of the pylon)
	roll=cactus_get_sc_roll(index[min(time)],pylon=pylon)
     if ((daReal lt 15) and (paReal gt pylon-15) and (paReal lt pylon+15)) then begin
        cmes[pos]=0
        good[icme]=0
     endif
     ;remove all cmes that don't start within the range
     ;[juldays[0],juldays[1]]
     if(start_index[icme] ge 0) then begin
        ;start_index exist so we use it
        this_cme_si = start_index[icme]
     endif else begin
        ;start_index doesn't exist, so we use the positionsbased method
        positions=where(cmes eq icme, npositions)
        time_positions = positions/(nangles*2)
        this_cme_si = time_positions[0]
     endelse
     if(this_cme_si lt juldays_start_pos or this_cme_si gt juldays_end_pos) then begin
        cmes[pos]=0
        good[icme]=0
     endif
      endfor
  cmes=label_region(cmes,/ALL_NEIGHBORS)
  ncmemap=max(cmes)
 
  ;adapt the old start_index, end_index, comment and save it again
  good_pos = where(good eq 1, ngood_pos)
  if ngood_pos gt 0 then begin
     start_index=start_index[where (good eq 1)]
     end_index=end_index[where(good eq 1)]
     comment=comment[where(good eq 1)]
  endif

  save,filename=dir.work+'startingtime.sav',start_index,end_index,comment

    ; renumber the remaining CMEs 
  cmemap = cmemap*0
  FOR cmei = 1, ncmemap DO BEGIN
     ;stop
     where_left = where(cmes[0:nangles-1,*] EQ cmei,nleft)
     where_right = where(cmes[nangles:*,*] EQ cmei,nright)
      
     ;cmemap=cmes[0:nangles-1,*]+cmes[nangles:*,*]
     IF nleft GT 0 THEN cmemap[where_left]=cmei
     IF nright GT 0 THEN cmemap[where_right]=cmei
  ENDFOR
  
  
  ;cactus_starttime,no_logfile=no_logfile , instrument=instrument
  ;restore, dir.work+'startingtime.sav' ;start_index and comment
  
        printf,log, '   CMEmap made'
 
    ; determine speed profiles within cmemap
      maskline   =BYTARR(nimages)
      mask       =BYTARR(nimages,nspeeds)
      myvelocity =FLTARR(nangles,nimages)
      fov_in_km = !cactusfov.nradii*!cactusfov.km_per_pix	;nradii=225, km_per_pix(in cactusmap)=100000
      for icme=1,ncmemap  do begin
	     for angle=0,nangles-1 do begin
	      thiscme=where(cmemap[angle,*] eq icme, nthiscme)
	      IF nthiscme gt 0 THEN BEGIN
               mask       =BYTARR(nimages,nspeeds)
               maskline[*]=0  &  maskline[thiscme]=1
               for ispeed=0, nspeeds-1 do mask[*,ispeed]=maskline
               tvangle=tvanglecube[*,*,angle]*mask
	       peak=max(tvangle,peakpos)
	       ypeakpos=peakpos/nimages 	    	;DT-pos
	       xpeakpos=peakpos-ypeakpos*nimages	;time-pos
	       DT = minDT+ ypeakpos ;
	       t0 = index[xpeakpos].obstime
	       IF peak GT 0 THEN BEGIN
	        IF xpeakpos +DT LT (nimages -1) THEN BEGIN
	          t1 = index[xpeakpos+DT].obstime
	          time_needed_to_cross_fov_in_sec = (t1-t0)*24.*60.*60.
                ENDIF ELSE BEGIN  ; in case CME is to slow, calculate virtual time_needed_to_cross_fov_in_sec
	          t1 = index[nimages-1].obstime
		  delta_t = xpeakpos+DT-nimages+1 & last_cadence=index[nimages-1].obstime-index[nimages-2].obstime
	          time_needed_to_cross_fov_in_sec = (t1-t0 + last_cadence*delta_t)*24.*60.*60.
                ENDELSE
	        v  = fov_in_km/time_needed_to_cross_fov_in_sec ; in km/s
 	       ENDIF ELSE v=0
               myvelocity[angle,xpeakpos] = v
	      ENDIF
	     endfor
      endfor

   ;free memory
   tvanglecube=0
   
  save, filename=dir.work+'CMEmap.sav',cmemap,cmes,tangle

        printf,log, '   velocities calculated'

     ; output
  ;printf,log, '   > start cactus_starttime'
  ;cactus_starttime,no_logfile=no_logfile , instrument=instrument
  ;printf,log, '   < cactus_starttime done'
        
  ;restore, dir.work+'startingtime.sav' ;start_index and comment
      ; calculate t0,dt0,pa,da

	first=FLTARR(ncmemap+1,2)
	pada=FLTARR(ncmemap+1,2)
        restore, dir.var+'cactuscatalog.sav'     ;this contains the structure of CME entries, defined in var.pro
	restore, dir.var+'julstart.sav'
	cmeindex=cactuscatalog
	cmecat=cmeindex
	cmecat[0].nr=0 & cmecat[0].type=' '
	
  
	
  for i=1,ncmemap  do begin   ; zero is background
	   positions=where(cmes eq i, npositions)   ;positions in the map: nonedgehit+edgehit
           time_positions  = positions/(nangles*2)
           angle_positions = (positions -time_positions*(nangles*2)) 
           first[i,1]=time_positions[0] & first[i,0]=(angle_positions[0] mod 180)*sf
	   left=min(angle_positions) 	  & right=max(angle_positions)
	 ; da & pa in real coordinates 
           da=min([(right-left)*sf,360]) 	    
           pa=(((left+right)/2.) mod nangles)*sf
           paReal=(pa+!cactusfov.edge-90) MOD 360          ;!cactusfov.edge is measuerd from mathematical theta=0 (=solar west)
	
           firstobstime=obstime(first[i,1])
           last = fix(round((obstime(max(time_positions)<(nimagesp2-3))-firstobstime)*24)) ; uncertainty interval
	    	    	    ; ????? how comes time_positions can be > nimagesp2-2 ???
	   ;firsttime=index[first[i,1]].date_obs+' '+STRMID(index[first[i,1]].time_obs,0,5)
	   if start_index[i] ge 0 then begin
	      firsttime=index[start_index[i]].date_obs+' '$
	      +STRMID(index[start_index[i]].time_obs,0,5) 
	      firstobstime=obstime[start_index[i]]
    	   endif else firsttime=STRTRIM(comment[i],2)+'   '

	   pos=where(cmemap eq i, npositions)   ;positions in the cmemap
	   cmevelocities=myvelocity[pos]
	   good=where(cmevelocities GT 0,ngood)
	  
	   case 1 of
	    (da lt 90): width=' '
	    (da ge 90): width='*'
	   endcase 

	   IF ngood GT 1 THEN  BEGIN
              cmevelocities=cmevelocities[good]
              cmevelocities=FIX(cmevelocities)
    	      ;calculate min and max velocity which are no outliers
    	      med=median(cmevelocities,/even)
    	      groupA=where(cmevelocities le med)
    	      groupB=where(cmevelocities ge med)
    	      Q1=median(cmevelocities[groupA])
    	      Q3=median(cmevelocities[groupB])
    	      interq=Q3-Q1
    	      rest=where((cmevelocities ge Q1-interq*1.5) and cmevelocities le Q3+1.5*interq,nr)
    	      M0=min(cmevelocities[rest]) & M1=max(cmevelocities[rest])
	      slow=M0 & fast=M1
    	      av_vel=median(cmevelocities[rest],/even)
	      sigma=sqrt(total((cmevelocities[rest]-av_vel)^2)/nr)
	   ENDIF ELSE BEGIN
	      av=1
	      sigma= 0
	      slow=1
	      fast=1
	      av_vel =1
	   ENDELSE
       ; make cme-structure
       ; the cmeindex.nr will be filled in by the track routine, 
       ; flows near occulter have been removed ans marginal cases 
       ; are not tracked.
     if start_index[i] ge 0 then begin
        cmeindex.date       =index[start_index[i]].date_obs
     endif else cmeindex.date       = index[first[i,1]].date_obs
     if start_index[i] ge 0 then begin
        cmeindex.time = index[start_index[i]].time_obs
     endif else cmeindex.time = index[first[i,1]].time_obs
     if start_index[i] ge 0 then begin
        cmeindex.jultime = obstime[start_index[i]]
     endif else cmeindex.jultime = obstime(first[i,1])
     if end_index[i]   ge 0 then begin
        cmeindex.enddate = index[end_index[i]].date_obs 
     endif else cmeindex.enddate = index[first[i,1]].date_obs
     if end_index[i]   ge 0 then begin 
        cmeindex.endtime = index[end_index[i]].time_obs 
     endif else cmeindex.endtime = index[first[i,1]].time_obs
     if end_index[i]   ge 0 then begin 
        cmeindex.endjultime = obstime[end_index[i]] 
     endif else cmeindex.endjultime = obstime(first[i,1])
                                      cmeindex.dt0        =last
                                      cmeindex.angle      =paReal
                                      cmeindex.width      =da
                                      cmeindex.speed      =FIX(ROUND(av_vel))
                                      cmeindex.speedsigma =sigma
                                      cmeindex.min_speed  =slow
                                      cmeindex.max_speed  =fast
     if start_index[i] ge 0 and sigma ne 0 then begin
        cmeindex.type       ='CME' 
     endif else cmeindex.type ='Flow' ; = marginal cases
    ; insert info on detector etc.
         cmeindex.obsrvtry   = index[first[i,1]].obsrvtry
         cmeindex.instrument = instrument
         cmeindex.detector   = cactus_detectors(instrument)

    ; add this event to the cmecat
         printf, log,'     Add ',cmeindex.type,' ',i,' to cmecat_struct'
	 cmecat=[cmecat,cmeindex]                ;cmecat[0] is empty, first entry is in cmecat[1]
	 infoline=cactus_infoline(cmeindex)             ;infoline is a FUNCTION
    ; insert info on detector etc.
         cmeindex.obsrvtry   = index[first[i,1]].obsrvtry
         cmeindex.instrument = instrument
         cmeindex.detector   = cactus_detectors(instrument)

      ; test if halo alert has to be sent  
	 ; calculate obstime of now
           restore, dir.var+'julstart.sav'
	   now=systime(/julian)
           obstime_now= now- float(julstart.int) -float(julstart.frac)
       ; test + collect info for mailing
           IF ((cmeindex.width ge 180) and $                 	;test1: halo?
	       (STRTRIM(cmeindex.type,2) eq 'CME') and $     	;test2: not a Flow (i.e. not marginal case)?
	       (cmeindex.speedsigma ne 0) and $              	;test3: sigma=0 is sign of false detection
	       (mailing_allowed) and $			     	;test4: this machine = operational machine?
	       (obstime_now-cmeindex.jultime LE 3)) THEN BEGIN	;test5: time of halo is not longer ago than 3 hours from now?
	     mail_line=[mail_line,strmid(infoline,5,56)]
	     time_line=[time_line,cmeindex.jultime]
	     printf, log, '   Infoline added to mail_line: '+strmid(infoline,5,56)
	   ENDIF  

	   pada[i,*]=[cmeindex.angle,cmeindex.width]
  endfor
        
      ;;store minimum year and month present in current run
      ; if ncmemap gt 0 then begin
      ;t=min([cmecat[1:*].jultime],m)
      ;yyyy=strmid(cmecat[m+1].date,0,4)                                       ;minimum year and month present in current run
      ;  mm=strmid(cmecat[m+1].date,5,2)
      ;cmecat[0].date=yyyy+'/'+mm
      ;endif

      ; sort CMEs and flows in time
	right_order=sort(cmecat.jultime)
        cmecat=cmecat[right_order] & first=first[right_order,*] & pada=pada[right_order,*]

        sortmap=cmemap
 	for i=1,max(cmemap) do begin 
           pos=where(cmemap eq right_order[i], npositions)   ;positions in the cmemap
           sortmap[pos]=i
	endfor 
	cmemap=sortmap

      ; give all CMEs a number
         cme=where(cmecat.type eq 'CME',n_cme)
         IF n_cme  gt 0 THEN  cmecat[cme].nr=1+indgen(n_cme)

       ; give all FLOWs a number 
         flow=where(cmecat.type eq 'Flow', n_flow)
	 IF n_flow gt 0 THEN cmecat[flow].nr=1+indgen(n_flow)
  
        ;check if CACTUSMAILER  has to be started
        IF n_elements(mail_line) gt 1 THEN BEGIN
	  mail_line=mail_line[1:*] & time_line=time_line[1:*]
	  cactus_mailer, mail_line, time_line,no_logfile=no_logfile,message=message ;message is output, but not used now		    
          printf, log, '   Message sent to cactus_mailer: '+strtrim(n_elements(mail_line),2)+' CME entries'
        ENDIF ELSE printf, log, '   No message sent to cactus_mailer'
	
     ; write data output
       good=where(myvelocity GT 0, ngood)
       IF ngood eq 0 THEN BEGIN
          printf, log, ' '
	  printf, log, '   Problem encountered: ngood =0; no cmemap at all?!'
       ENDIF

       save,filename=dir.work+'CMEmap.sav',cmemap,myvelocity,first,last,tangle,cmes,pada
       save,filename=dir.out+'cmecat_struct.sav',cmecat       ;structure with cme info

   jump:
       
       
; ------------------------closing remarks-------------------------------
 cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END
