;+
; NAME:
;    cactus_classify_region
;
; PURPOSE:
;    This function substacts the regions and creates features of every 
;    region. These feature are min angle, max angle, min radius, max
;    radius, image number, regionsnumber
;
;
; USE:
;     result = cactus_classify_region(regions)
;
; INPUTS:
;     parameters:
;         regions:    the regions to classify
; OUTPUTS:
;     An array of structures of the form:
;         {imagenumber,
;          regionnumber,
;          minangle,
;          maxangle,
;          minradius,
;          maxradius}
;
; MODIFICATION HISTORY
;    2010/08/12, BB, Created
;==========================================================
;-
FUNCTION cactus_classify_region, regions
  classified_region = {$
                       imagenumber:0,$
                       regionnumber:0,$
                       minangle:0,$
                       maxangle:0,$
                       minradius:0,$
                       maxradius:0,$
                       numberofpixels:0$
                      }

  cl_region_list = classified_region
  nimages = (size(regions))[3]
  nrad = (size(regions))[2]
  nang = (size(regions))[1]
  
  FOR imind = 0, nimages-1 DO BEGIN
     hist = histogram(regions[*,*,imind])
     FOR histi = 1,N_ELEMENTS(hist)-1 DO BEGIN
        places = where(regions[*,*,imind] EQ histi,nplaces)
        IF nplaces GT 0 THEN BEGIN
           angles = places MOD nang
           radius = places / nang
           minangle = min(angles)
           maxangle = max(angles)
           minradius = min(radius)
           maxradius = max(radius)
           temp = classified_region
           temp.imagenumber = imind
           temp.regionnumber = histi
           temp.minangle = minangle
           temp.maxangle = maxangle
           temp.minradius = minradius
           temp.maxradius = maxradius
           temp.numberofpixels = nplaces
           ;stop
           cl_region_list = [cl_region_list,temp]
        ENDIF
     ENDFOR
  ENDFOR
  
  IF N_ELEMENTS(cl_region_list) gt 1 THEN BEGIN
     RETURN,cl_region_list[1:*]
  ENDIF ELSE BEGIN
     RETURN, -1
  ENDELSE

END
