;======================== header ===============================================
;+
; NAME: 
;   cactus_cmemap
;    
; PURPOSE:
;
; CATEGORY:
;   cactus 
; INPUTS:
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;   /crontab: set this keyword when working in the background. In this case,
;             the 'Z' buffer is use for plotting, as no real screen is
;             available.
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   
; MODIFICATION HISTORY:
;   2002/11/20, ver 1.00, David Berghmans
;   2003/06/11, DB, adapted to the combined c2/c3 format
;   -------------   version 2.0.0 frozen here   -----------
;   2005/11/30, ER, made compatible with the number tracking  
;   2006/02/16, ER, add flow numbers on map
;   2006/02/23, ER, cmecat.nr instead of numbers.sav
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/03/12, ER, remove crontab keyword, always use Z-buffer for images
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;-
;======================== end header ===========================================
PRO cactus_cmemap, cmecat,no_logfile=no_logfile

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------
 ; input
   restore, dir.work+'index.sav'
   obstime   = index.obstime
 
   restore, dir.work+'CMEmap.sav' ; cmemap, myvelocity, first, last
   sz        = size(cmemap,/dimensions)
   ncmes     = max(cmemap)
   
 ; determine graphics device
   outdimx=sz[0]*2+90	;90 is space to add the date&time info
   outdimy=sz[1]+22
   printf, log, '   Working in batchmode, using Z-buffer'
   set_plot, 'Z'
   device, set_resolution = [outdimx,outdimy]   ; size of image
   device, z_buffering    = 0           ; 0 for 2D

 ; prepare for screen output
   ;device, decomposed=0
   tvlct, myr,myg,myb,/get
   RED  = [255,255 ,    0,    0,   0,  255]	       ;  red=1,  green=2
   GREEN= [0  , 255,   255,  255,   0,    0]	       ; blue=3, yellow=4, purple=5
   BLUE = [0  ,  0,     0,  255,  255, 255]
   myr[0:5]=RED
   myg[0:5]=GREEN
   myb[0:5]=BLUE
   tvlct, myr,myg,myb
  
; write png output, cme color corresponds to cmecat.nr
   restore, dir.out+'cmecat_struct.sav' ; cmecat
   background=bytarr(outdimx,outdimy)
   background[*]=7
   tv, background
   new_cmemap=cmemap*0
   for i=1,ncmes do $
   	if strtrim(cmecat[i].type,2) eq 'CME' then new_cmemap[where(cmemap eq i)]=cmecat[i].nr  $
	else new_cmemap[where(cmemap eq i)]=-1
   tv, congrid((new_cmemap EQ 0)*7 + new_cmemap MOD 4 +(new_cmemap eq -1) MOD 5,sz[0]*2,sz[1]) 
              ; 'backgrnd colors 7; CME colors  0 to 3; flows color 4 (4=(-1 mod 4) + (1 mod 5))

   save, filename=dir.work+'new_cmemap.sav',new_cmemap   

   for i=0,sz[1]-13,15 do xyouts,362,i,STRMID(index[i].time_obs,0,5)+' '+STRMID(index[i].date_obs,5,5) ,/device
   ; make sure that the last corresponds indeed to the last
   i=sz[1]-1
   xyouts,362,i,STRMID(index[i].time_obs,0,5)+' '+STRMID(index[i].date_obs,5,5) ,/device
  
   plots, [0,0],[0,sz[1]+22],color=db_color('green'), /device
   plots, [36,36],[0,sz[1]+22],color=db_color('red'), /device 
   plots, 36+[90,90],[0,sz[1]+22],color=db_color('red'), /device, linestyle=3
   plots, 36+[180,180],[0,sz[1]+22],color=db_color('red'), /device 
   plots, 36+[270,270],[0,sz[1]+22],color=db_color('red'), /device, linestyle=3
   plots, [360,360],[0,sz[1]+22],color=db_color('green'), /device
   xyouts, 36    -6, 30,'S', color=db_color('red'), /device  , charsize=2
   xyouts, 36+90 -6, 30,'W', color=db_color('red'), /device  , charsize=2
   xyouts, 36+180-6, 30,'N', color=db_color('red'), /device  , charsize=2
   xyouts, 36+270-6, 30,'E', color=db_color('red'), /device  , charsize=2
 
   for i=1,ncmes do begin
     thiscme=cmecat[i]
     if strtrim(thiscme.type,2) eq 'CME' then $ 
     xyouts, first[i,0] , first[i,1], 'c'+STRTRIM(thiscme.nr,2),/device, charsize=1, color=db_color('red') $
     else $
     xyouts, first[i,0] , first[i,1], 'f'+STRTRIM(thiscme.nr,2),/device, charsize=1,   color=db_color('red') 
   endfor		      

   write_png, dir.out+'detectionmap.png', tvrd(),myr,myg,myb 

; ------------------------closing remarks-------------------------------
 
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END
