;======================== header ===============================================
;+
; NAME: 
;   cactus_cmemovie (old name = cmemovie2)
;    
; PURPOSE:
;
; CATEGORY:
;   cactus 
; INPUTS:
;   telescope: one of the following: 'c2', 'c3', 'C2', 'C3'
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;   /crontab: set this keyword when working in the background. In this case,
;             the 'Z' buffer is use for plotting, as no real screen is
;             available.
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   cactus_line_on_array.pro
;   strnr.pro
; MODIFICATION HISTORY:
;   2002/08/01, ver 1.00, David Berghmans
;   2002/08/23, DB, crontab keyword
;   2002/11/02, DB, telescope is now input argument
;   2003/06/04, DB, now telescope independent
;   2003/06/10, DB, pipe dir removed
;   2004/10/22, ER+BN, make mpeg movie + write date and time on the images
;   2004/10/29, ER, convert program uses too much memory, temporarily taken out!
;   -------------   version 2.1.0 -------------------------
;   2007/08/13, ER, make difference movie of whole c2 sequence using makeMov
;   -------------   version 2.2.0 -------------------------
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/01/17, ER, makemov is now an IDL routine
;   2008/01/18, ER, rewrite the velocity calculation
;   2008/01/22, ER, im???.jpeg replaced by im????.jpeg (i.e. need 4 digits for number instead of 3, typical number of images for 1 month is 2500)
;   2008/03/12, ER, Remove crontab keyword
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/05/23, ER, add keyword tempdir, usd dir.tmp when calling makemov inside cactus
;   2009/03/04, ER, add keyword no_movie to avoid making CACTus_movie.mov (polar transformed movie for whole dataset)
;   2009/05/22, ER, change keyword no_movie into movie: default is no_movie
; -
;======================== end header ===========================================
PRO cactus_cmemovie, no_logfile=no_logfile, thresh=thresh,movie=movie
   
   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile,  /open
 
; ------------------------ keyword settings ----------------------------
   IF NOT KEYWORD_SET(thresh)  THEN thresh = 0.3      ; set default intensity threshold

;-----------------------main code--------------------------
 
 ; open data files
   restore, dir.work+'CMEmap.sav'   ;cmemap 
   restore, dir.work+'datacube.sav'   ; datacube
   restore, dir.work+'index.sav'   ; index
   restore, dir.work+'tvanglecube.sav'
   restore, dir.work+'vectors.sav'
   restore, dir.work+'velocity_settings.sav' ; maxDT
   openw, listfile, dir.out+'list.txt', /get_lun
 
   tvanglecube_trans = 0
 ; main parameters  
   obstime=index.obstime
   nangles  = (size(datacube))[1]
   nradii   = (size(datacube))[2]
   nimages  = (size(datacube))[3]

   datacube = 0
      printf, log, '   Working in batchmode, using Z-buffer'
      set_plot, 'Z'
      device, set_resolution = [nangles*2,nradii*2]   ; size of image
      device, z_buffering    = 0           ; 0 for 2D

 ; prepare for screen output
   ;device, decomposed=0
   tvlct, myr,myg,myb,/get
   RED  = [255,255 ,    0,    0,   0,  255]	       ;  red=1,  green=2
   GREEN= [0  ,255,   255,  255,   0,    0]	       ; blue=3, yellow=4, purple=5
   BLUE = [0  ,  0,     0,  255,  255, 255]
   myr[0:5]=RED
   myg[0:5]=GREEN
   myb[0:5]=BLUE
   tvlct, myr,myg,myb
   

; calculate velocities for Cactus
  printf, log, 'Loop 1: Tracking velocities. Total samples=',nimages
  motion=intarr(nangles, nradii+1, nimages+maxDT)
  peakDT=max(tvanglecube,dim=2,peakDTpos)
  peakDT=transpose(peakDT)
  cmes=where(tangle*cmemap gt 0 AND peakDT  gt thresh,npoints)
  IF NPOINTS EQ 0 THEN GOTO, SKIP
  ind=array_indices(cmemap,cmes)
  angles=ind[0,*] & times=ind[1,*]
  FOR i=0,npoints-1 DO BEGIN
     pos=array_indices(tvanglecube,peakDTpos[times[i],angles[i]])
     motionslice=transpose(motion[angles[i],*,*])
     motionslice[vectors[*,pos[1],times[i]]]=CMEmap[angles[i],times[i]]
     motion[angles[i],*,*]=transpose(motionslice)
  ENDFOR  
  motion=motion[0:(nangles-1),0:(nradii-1),0:(nimages-1)]

  vectors=0
  tvanglecube=0
  

  SKIP:

;  now connect the individual dots in motion as lines representing a CME front
   ncmes=max(CMEmap)
   printf, log, 'Loop 2: Connecting dots '
   for i_image=1,nimages-1 do begin
      current_image=motion[*,*,i_image]
      loccmes  = where(current_image GT 0,nloccmes)
      IF nloccmes GT 0 THEN BEGIN
          first_cme_in_current_image=min(current_image[loccmes])
          last_cme_in_current_image=max(current_image[loccmes])
          for i_cme=first_cme_in_current_image,last_cme_in_current_image do begin
             a = current_image eq i_cme
             loc =where(a,nloc)
 	     IF nloc GT 2 THEN BEGIN
                ypos=loc/nangles
                xpos=loc-nangles*ypos
   
                ascending=sort(xpos)
                xpos=xpos[ascending]
                ypos=ypos[ascending]
   
                distance=sqrt((xpos[1:nloc-1]-xpos[0:nloc-2])^2+(ypos[1:nloc-1]-ypos[0:nloc-2])^2)
    		 
                for i=0L,nloc-2 do IF (distance[i] GT 2 AND distance[i] LT 10) THEN $
                  cactus_line_on_array, a, xpos[i],ypos[i], xpos[i+1],ypos[i+1], i_cme
   	        motion[*,*,i_image] = motion[*,*,i_image] > a
             ENDIF 
          endfor
      ENDIF	  
   endfor   
   
   restore, dir.work+'datacube.sav'   ; datacube

 ; write image per image
   nimages=(size(datacube))[3]
   dimx=(size(datacube))[1]
   dimy=(size(datacube))[2] 
   bigimage=bytarr(dimx*2,dimy*2)
   
   previous_time=0
   printf, log, 'Loop 3: Creating images '
   for imind=0, nimages-1 do begin
   
      image=datacube[*,*,imind]
      zeros= where(total(image,1) EQ 0,nzeros)

      image= bytscl(image<0.03>(-0.03), top=246)+7
      ori_image=image
      thismotion=motion[*,*,imind]
      cmes=where(thismotion GT 0, ncmes)
      IF ncmes GT 0 THEN image[cmes]=(thismotion[cmes] MOD 5)


      IF (index[imind].obstime-previous_time LT 0.1) and nzeros GT 0 THEN BEGIN
         FOR i = 0,nzeros-1 do ori_image(*,zeros[i])=old_ori_image(*,zeros[i])
	 FOR i = 0,nzeros-1 do image(*,zeros[i])=old_image(*,zeros[i])
      ENDIF
      ori_image_360=rebin(ori_image,360,dimy)
      image_360    =rebin(image,360,dimy)
      bigimage[*,dimy:*]    =ori_image_360   ; top half is original image
      bigimage[*,0:(dimy-1)]=image_360   ; bottom half is original image
                                       ;  + detections superimposed
 
      tv, bigimage
      dateline=index[imind].date_obs+' '+STRMID(index[imind].time_obs,0,8)
      xyouts, 10,dimy,dateline,color=db_color('red'),charsize=1,/device
      write_image,  dir.images+'im'+STRNR(imind,4)+'.jpeg','JPEG',tvrd(),myr,myg,myb
      printf,listfile,'im'+STRNR(imind,4)+'.jpeg'+$
	        ' "'+dateline+'"'
		
      previous_time= index[imind].obstime		
      old_image    = image
      old_ori_image= ori_image
      
    endfor

 ; close file
    close, listfile		
    printf,log, 'Close listfile'

 ; make LASCO C2 difference movie
    ;printf, log, ' Make CME.mov'
    ;spawn, dir.bin+'makeMov '+dir.images+' im '+dir.bin+'ffmpeg'

 IF KEYWORD_SET(movie) THEN BEGIN
    printf, log, ' Call makemov.pro'
    moviename=dir.out+'CACTus_movie.mov'
    success=makemov(dir.images,infiles='im*.jpeg',tempdir=dir.tmp+'cmemovie/',outfile=moviename,ffmpegdir=dir.bin,ffmpegoptions="-r 6")
    IF success THEN printf, log, ' Movie was made successful: ',moviename $ 
    ELSE printf, log, ' ATTENTION! Problem with making movie: ',moviename
 ENDIF

 ; log   
   printf, log, '   new cme movie made'

 ; ------------------------closing remarks-------------------------------
   cactus_loginfo, myname, log, no_logfile=no_logfile, /close
   
END

