;+
; NAME:
;   cactus_correct_polar
;
; PURPOSE:
;   This function corrects the given polarcube based on the streakscube.
;
; USE:
;   cleaned_polar = cactus_correct_polar(
;                                        polar,
;                                        streaks
;                                       ) 
;
; INPUTS:
;     parameters:
;         polar:          The cube with the original (!not the
;                         difference images) 
;
;         streaks:        The cube with the streaks.
;
; OUTPUTS:
;     The corrected polarcube
;
;
; MODIFICATION HISTORY:
;    2010/08/12, BB, Created
;-
FUNCTION cactus_correct_polar,streaks,polar
  nimages = (size(polar))[3]
  nrad = (size(polar))[2]
  nang = (size(polar))[1]
  corr_polar = polar
  FOR imind = 0, nimages-1 DO BEGIN
     veeg_pos = WHERE(streaks[*,*,imind] GE 1,nveeg_pos)
     IF nveeg_pos GT 0 THEN BEGIN
        polimage = polar[*,*,imind]
        IF (imind GT 1) AND (imind LT nimages-2) THEN BEGIN
           ;most frequent case
           prev_prev_pol = polar[*,*,imind-2]
           prev_pol = polar[*,*,imind-1]
           fol_pol = polar[*,*,imind+1]
           fol_fol_pol = polar[*,*,imind+2]
        ENDIF ELSE BEGIN
           IF imind EQ 0 THEN BEGIN
              prev_pol = polar[*,*,imind+1]
              prev_prev_pol = polar[*,*,imind+2]
              fol_pol = polar[*,*,imind+1]
              fol_fol_pol = polar[*,*,imind+2]
           ENDIF ELSE BEGIN
              IF imind EQ nimages-1 THEN BEGIN
                 prev_pol = polar[*,*,imind-1]
                 prev_prev_pol = polar[*,*,imind-2]
                 fol_pol = polar[*,*,imind-1]
                 fol_fol_pol = polar[*,*,imind-2]
              ENDIF ELSE BEGIN
                 IF imind EQ 1 THEN BEGIN
                    prev_prev_pol = polar[*,*,imind-1]
                    prev_pol = polar[*,*,imind-1]
                    fol_pol = polar[*,*,imind+1]
                    fol_fol_pol = polar[*,*,imind+2]
                 ENDIF ELSE BEGIN
                    IF imind EQ nimages-2 THEN BEGIN
                       prev_prev_pol = polar[*,*,imind-2]
                       prev_pol = polar[*,*,imind-1]
                       fol_pol = polar[*,*,imind+1]
                       fol_fol_pol = polar[*,*,imind+1]
                    ENDIF ELSE BEGIN
                       MESSAGE, 'WHOOHOO you did the impossible'
                    ENDELSE
                 ENDELSE                  
              ENDELSE
           ENDELSE
        ENDELSE
        polimage[veeg_pos] = (prev_pol[veeg_pos]+fol_pol[veeg_pos])/2
        corr_polar[*,*,imind] = polimage
     ENDIF
  ENDFOR
  RETURN, corr_polar
END
