;======================== header ===============================================
;+
; NAME:
;   cactus_dark_blocks
; PURPOSE:
;   Remove pixels that are nog officially missing, but too dark in the time series
;
; CATEGORY:
;   cactus
;
; INPUTS:
;   polar
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   
; MODIFICATION HISTORY:
;   2008/03/18, ER, Created, extracted from cactus_reldev
;-
;======================== end header ===========================================
PRO cactus_dark_blocks, polar, no_logfile=no_logfile

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------
  sz=size(polar)
  npixel1=sz[1]
  npixel2=sz[2]
  nimages=sz[3]
   
  badimages=intarr(nimages)
  for x=0,npixel1-1 do begin
   for y=0,npixel2-1 do begin
	prof=polar[x,y,*]
	stat=moment(prof)
	av=stat(0)
	sigma=sqrt(stat(1))
        bad= where(prof LT av - 5*sigma,nbad)
	IF nbad gt 0 then begin
	   prof[bad]=0
	   badimages[bad]=badimages[bad]+1
	endif	 
	polar[x,y,*]=prof
    endfor; y
   endfor; x
   bad= where(badimages GT 0, nbad)
   IF nbad gt 0 then BEGIN
     printf, log, '      Image nr | nr. dark pixels'
     printf, log, '      --------------------------'
     for i=0, nbad-1 do printf, log,  bad[i],'   |', badimages[bad[i]]
   ENDIF
	    
   ; extend the missing blocks a bit
   mb     =BYTARR(npixel1,npixel2,nimages)  ; to store missing blocks
   s      =replicate(1b,5,5)
   FOR i=0,nimages-1 DO mb[*,*,i]= dilate(polar[*,*,i] LE 0,s)

; ------------------------closing remarks-------------------------------
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END 
