;======================== header ===============================================
;+
; NAME: 
;   cactus_diffmovie
;    
; PURPOSE:
;
; CATEGORY:
;   cactus ver 3.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; INPUTS:
;   telescope: one of the following: 'c2', 'c3', 'C2', 'C3'
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;
; CALLS:
;   cactus_cme_txt
;   strnr
;
; COMMON BLOCKS: 
;   none
;
; MODIFICATION HISTORY:
;   2002/11/27, ver 1.00, David Berghmans
;   2003/01/02, DB, renamed (formerly LASCOmovie, now diffmovie)
;   2003/01/10, DB, xycube=float(xycube) avoids discretisation errors
;   2003/01/17, DB, output names changed
;   2003/06/10, DB, pipe dir removed
;   2004/09/10, DB, color table has changed for consistency with SWB
;   2004/10/22, ER+BN, make mpeg movie + write date and time on the images
;   2004/10/29, ER, convert program uses too much memory, temporarily taken out!
;   2004/12/21, ER, make images and movie-list for each cme (uses AniS applet)
;   2005/02/16, ER, If no c2 images available for a particular CME, copy c2_no_im.jpeg from cactus/out/ to cactus/out/cme_i
;   -------------   version 2.0.0 frozen here   -----------
;   2005/12/01, ER, made compatible with the number tracking
;   2005/12/01, ER, added bow and comment on middle figure
;   -------------   version 2.1.0 -------------------------
;   2007/08/13, ER, make difference movie of whole c2 sequence using makeMov
;   -------------   version 2.2.0 -------------------------
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/01/16, ER, makemov is now and IDL function
;   2008/03/12, ER, remove crontab keyword
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/24, ER, replace telescope with detector and use cactus_detectors to find the detector
;   2008/04/03, ER, add keyword no_movie (movie for whold dataset in xy coordinates)
;   2008/05/23, ER, add keyword tempdir, usd dir.tmp when calling makemov inside cactus
;   2009/04/14, ER, apply exposure time fluctuations here (only for LASCO at the moment) and use rundiff_cube.pro to make running difference images
;   2009/04/14, ER, change syntax in bytscl, also change values to -5, 5 for LASCO
;   2009/05/22, ER, change keyword no_movie into movie: default is no_movie
;-
;======================== end header ===========================================
PRO cactus_diffmovie , no_logfile=no_logfile , movie=movie, instrument=instrument
   
  cactus_preamble, dir,myname
  cactus_loginfo, myname, log, no_logfile=no_logfile, /open

;-----------------------main code--------------------------
 ; sanity check
   detector=(cactus_detectors(instrument))[0] 
   imlabel=instrument+'-'+detector
   printf, log, ' detector=', detector
 
 ; open data files
   restore, dir.work+detector+'_xy_images.sav'   ; xycube
   restore, dir.work+detector+'index.sav'   ; index
   restore, dir.var+'julstart.sav';julstart

   openw, listfile, dir.out+detector+'list.txt', /get_lun
 
 ; main parameters  
   npixelx  = (size(xycube))[1]
   npixely  = (size(xycube))[2]
   nimages  = (size(xycube))[3]

 ; determine graphics device
   ;device, decomposed=0
      printf, log, ' Working in batchmode, using Z-buffer'
      set_plot, 'Z'
      device, set_resolution = [npixelx,npixely]   ; size of image
      device, z_buffering    = 0           ; 0 for 2D

 ; prepare for screen output
   ;device, decomposed=0
   tvlct, myr,myg,myb,/get
   RED  = [255,255 ,    0,    0,   0,  255]	       ;  red=1,  green=2
   GREEN= [0  ,255,   255,  255,   0,    0]	       ; blue=3, yellow=4, purple=5
   BLUE = [0  ,  0,     0,  255,  255, 255]
   myr[0:5]=RED
   myg[0:5]=GREEN
   myb[0:5]=BLUE
   tvlct, myr,myg,myb
   
 ; apply correction for exposure time fluctuations
   xycube=float(xycube)
   if instrument eq 'LASCO' then begin
     restore, dir.work+detector+'_remetf.sav' ; yields timeline
     for i=0, nimages-1 do xycube[*,*,i]=xycube[*,*,i]/timeline[i]
   endif 

 ; write image per image
   printf, log, ' Make images in diffcube '
   diffcube=rundiff_cube(index,xycube) 

   for imind=0, nimages-1 do begin
      image=diffcube[*,*,imind]
      CASE instrument OF
	'LASCO': image= bytscl(image,-10,10, top=246)+7
	'SECCHI': image=bytscl(image,-2,2, top=246)+7
      ENDCASE
      tv, image
      caldat, double(julstart.int+julstart.frac)+index[imind].obstime, mm,dd,yyyy,ho,mi,se    
      yy=STRMID(STRTRIM(yyyy,2),2,2) & mm=STRNR(mm,2) & dd=STRNR(dd,2) 
      ho=STRNR(ho,2) & mi=STRNR(mi,2)& se=STRNR(se,2) 
      im=imlabel+'_'+yy+mm+dd+'.'+ho+mi+se
      dateline=index[imind].date_obs+' '+STRMID(index[imind].time_obs,0,8)
      xyouts, 5,5,dateline,color=db_color('red'),charsize=0.8,/device
      xyouts, 180,3,imlabel,/device, charsize=0.8
      write_jpeg,  dir.images+im+'.jpeg',tvrd(),quality=100
      printf,listfile,im+'.jpeg "'+dateline+'"'   
    
    ; prepare for next cycle
 ;;;     previous_image=current_image
 ;;;     diffcube[*,*,imind]=image		
    
    endfor

 ; close file
    close, listfile		
    free_lun, listfile

 IF KEYWORD_SET(movie) THEN BEGIN
    printf, log, ' Call makemov.pro'
    moviename=dir.out+imlabel+'_movie2.mov'
    success=makemov(dir.images,infiles=imlabel+'*.jpeg',tempdir=dir.tmp+'diffmovie/',outfile=moviename,ffmpegdir=dir.bin,ffmpegoptions="-r 6") 
    IF success THEN printf, log, ' Movie was made successful: ',moviename $ 
    ELSE printf, log, ' ATTENTION! Problem with making movie: ',moviename
 ENDIF 
 ; ------------------------closing remarks-------------------------------
   
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END

