;======================== header ===============================================
;+
; NAME:
;   cactus_download_lasco 
; PURPOSE:
;   gets several days of lasco images from data repository
; 
; CATEGORY:
;   cactus 
; INPUTS:
;   julday = an array containing the start- and enddate in julian calendar (is given by jul_startend)
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;   instrument  'c2' or 'c3'     
;
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   cactus_loginfo 
;   cactus_preamble
;   cactus_startend   
;     
; TO DO: 
;     
;     
; MODIFICATION HISTORY:
;   2008/03/13, ER, Created (extracted from cactus_getdata.pro)
;   2008/03/17, ER, Replace 'instrument' with 'detector' this is the name of 'c2' and 'c3' and 'COR2' in the header
;   2008/04/02, ER, Do not download data, but only a list of images containing the full path
;   2008/05/16, JMK, Check if file exist before addding to goodfiles
;-  2009/11/09, ER, Fix check for prepped files: the listofimages is checked against files listed in dir.in+detector 
;========================== end header =====================================================
PRO cactus_download_lasco, juldays, detector=detector, no_logfile=no_logfile

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile,  /open

; ------------------------main code ------------------------------------

   total_new_nimages=0
   CASE detector OF
     'c2' : filter='Orange'
     'c3' : filter='Clear'
   ENDCASE

   printf, log, ' Detector : ', detector
   image_list=' '
   FOR julday=juldays[0],juldays[1] DO BEGIN
    ; construct the 'day directory name' using nrpastdays
      caldat,julday, mm,dd,yyyy
      day=STRMID(STRNR(yyyy,4),2,2)+STRNR(mm,2)+STRNR(dd,2)
   
       ; get img_hdr.txt to see which images we need to download
         sourcefile    = dir.data+day+'/'+detector+'/'+'img_hdr.txt'
         IF file_test(sourcefile) THEN BEGIN
          ; read img_hdr.txt
            openr,dlu,sourcefile,/get_lun
            listofimages=' '
            line=' '
            REPEAT BEGIN
               readf,dlu,line
               listofimages=[listofimages, line]
            ENDREP UNTIL (eof(dlu))
            close,dlu & free_lun,dlu
            listofimages=listofimages[1:*]

          ; select images which need to be downloaded
            nimages=n_elements(listofimages)
            downloads=' '
            for i=0, nimages-1 do begin
               words=strsplit(listofimages[i],' ',/extract)
               IF n_elements(words) GE 12 THEN BEGIN
                  file=strsplit(words[0],'.',/extract)	
                  filename=file[0]+'_cactus.xdr.gz'
                  not_seen_yet = not file_test(dir.in+detector+path_sep()+filename)
               ;IF n_elements(words) LT 12 THEN stop                 
                  newfile=dir.data+day+path_sep()+detector+path_sep()+words[0]
                  good         =  (words[9] EQ filter) and (words[10] EQ 'Clear') and (words[11] EQ 'Normal') and (file_exist(newfile))
               ENDIF ELSE good = 0
               IF not_seen_yet and good THEN image_list=[image_list, newfile]
            endfor
         ENDIF ELSE printf, log, ' Could not open '+sourcefile

  ENDFOR ; juldays

  if n_elements(image_list) eq  1 then  printf, log, ' NO IMAGES ARE FOUND: CHECK YOUR INPUT DATES' $
  else begin
 	image_list=image_list[1:*]
  	printf, log, ' Number of images that will be downloaded: ',n_elements(image_list) 
  	save, filename=dir.work+detector+'_image_list.sav', image_list
  	printf, log, image_list
  endelse

; ------------------------closing remarks-------------------------------
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close


END 
