;======================== header ===============================================
;+
; NAME:
;   cactus_download_secchi 
; PURPOSE:
;   saves the output of scclister in cor2_image_list.sav 
; 
; CATEGORY:
;   cactus 
; INPUTS:
;   julday = an array containing the start- and enddate in julian calendar (is given by jul_startend)
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;   sc		'a' or 'b' (incl. quotes)
;   instrument  'COR2'     
;
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   cactus_loginfo
;   cactus_preamble
;   cactus_startend   
;
; TO DO:
;
;
; MODIFICATION HISTORY:
;   2008/03/13, ER, Created
;   2008/03/17, ER, Replace 'instrument' with 'detector' this is the name of 'c2' and 'c3' and 'COR2' in the header
;   2008/04/02, ER, Do not download the images, but save a stringarray containing the list of images. (copying data is too heavy for SECCHI)
;   2009/02/17, ER, Allow upper and lower case of keyword sc: change CASE statement into strlowcase(sc)
;   2009/08/18, ER, Only keep images every half hour (= nominal cadence of total brightness images)
;   2009/11/09, ER, Fix check for prepped files: the image_list is checked against files listed in dir.in+detector 
;-
;========================== end header =====================================================
PRO cactus_download_secchi, juldays, detector=detector, sc=sc, no_logfile=no_logfile

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile,  /open

; ------------------------main code ------------------------------------
 
   caldat,juldays[0], mm,dd,yyyy
   day1=STRNR(yyyy,4)+STRNR(mm,2)+STRNR(dd,2)
   caldat,juldays[1], mm,dd,yyyy
   day2=STRNR(yyyy,4)+STRNR(mm,2)+STRNR(dd,2)
   printf, log, ' Downloading day '+day1+' until '+day2
   list_struct=scclister(day1+'..'+day2, detector, sc=sc, size='2048', dest='SSR1')
   
   IF n_tags(list_struct) eq 0 THEN printf, log, ' NO IMAGES ARE FOUND: CHECK YOUR INPUT DATES' $
   ELSE BEGIN
     CASE strlowcase(sc) OF 
       'a' : image_list=list_struct.sc_a
       'b' : image_list=list_struct.sc_b
     ENDCASE

     ; Only keep images every half hour to keep cadence stable. When there is not enough contact with the ground, 
     ; STEREO makes total brightness images at hh:52 and hh:22 instead of polarization sequences. This unstable cadence influences the detection rate.
     times=anytim2cal(strmid(file_basename(image_list,'fts'),0,15),form=11) ;SebProg='Doub'  =: only doubles
     select=grid_data(times, min=30) 	; only keep images at a cadence of 30 min
     image_list=image_list[select] 

     ; select images which need to be downloaded
     nimages=n_elements(image_list)
     downloads=' '
     not_seen_yet=intarr(nimages)
     for i=0, nimages-1 do begin
    	word=file_basename(image_list[i])
	file=strsplit(word, '.',/extract)
	filename=file[0]+'_cactus.xdr.gz'
	not_seen_yet[i] = not file_test(dir.in+strlowcase(detector)+path_sep()+filename)
     endfor
     image_list=image_list(where(not_seen_yet MOD 2)) 
     if n_elements(image_list) le 15 then begin
	printf, log, ' # images is below 15, go to end of program'
	image_list=0
     endif
     printf, log, 'Detector  : ', detector
     printf, log, 'Spacecraft: ', sc
     printf, log, ' Number of images that will be downloaded: ',n_elements(image_list) 
     print, 'Number of images that will be downloaded: ',n_elements(image_list)
     save, filename=dir.work+strlowcase(detector)+'_image_list.sav', image_list
     printf, log, image_list

   ENDELSE
; ------------------------closing remarks-------------------------------
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close


END
