;======================== header ===============================================
;+
; NAME:
;   cactus_getdata (replaces old routine in.pro) 
; PURPOSE:
;   downloads the list of available images for the requested period
; 
; CATEGORY:
;   cactus 
; INPUTS:
;   
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   cactus_loginfo
;   cactus_preamble
;   cactus_startend   
;
; TO DO:
;
;
; MODIFICATION HISTORY:
;   2007/12/09, ER, created from in.pro
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2007/12/23, ER, same version for real-time or given dates
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir given as keyword, this allows to define a working directory
;   2008/01/09, ER, move 'rm '+daylistfile to the end of the routine
;   2008/01/10, ER, introduce dir.dataqkl and dir.datalz, this program takes lz is qkl is not available 
;   2008/01/11, ER, replace findfile by file_test routine. findefile is obsolite from IDL6.1 onwards
;   2008/01/30, ER, replace spawn commands with file_copy and file_delete
;   2008/03/10, ER, introduce dir.data to make it more generic. Instrument and lz or qkl is defined in cactus_settings 
;   2008/03/12, ER, Remove crontab keyword
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/13, ER, extract cactus_download_lasco.pro out ofcactus_getdata, also make cactus_download_secchi
;   2008/03/17, ER, Replace 'instrument' with 'detector' this is the name of 'c2' and 'c3' and 'COR2' in the header
;   2008/03/18, ER, replace the secchi keyword with instrument similar as in cactus_incube and cactus_reldev
;   2010/07/14, BB, juldays[0]-1, juldays[1]+1, save juldays
;-
;======================== end header ===========================================
PRO cactus_getdata, ndays=ndays, startdate, enddate, instrument=instrument, sc=sc, no_logfile=no_logfile

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile,  /open

; ------------------------main code ------------------------------------

   IF n_params() eq 0 THEN BEGIN
      juldays=jul_startend(ndays=ndays) 
      printf, log, ' We are in Real-Time mode'
   ENDIF ELSE IF n_params() eq 1 THEN BEGIN 
      juldays=jul_startend(ndays=ndays, startdate)
      printf, log, ' Startdate is given'
   ENDIF ELSE BEGIN
      juldays=jul_startend(ndays=ndays, startdate, enddate)
      printf, log, ' Startdate and enddate are given'
   ENDELSE
   
	save, filename=dir.work+'julstart_end.sav', juldays
	juldays[0]=juldays[0]-1
	juldays[1]=juldays[1]+1
   
   ; loginfo on startdate and enddate
   caldat,juldays, mm,dd,yyyy
   printf, log, ' First day is: ',STRNR(yyyy[0],4),'/',STRNR(mm[0],2),'/',STRNR(dd[0],2)
   printf, log, '  Last day is: ',STRNR(yyyy[1],4),'/',STRNR(mm[1],2),'/',STRNR(dd[1],2)

   detectors=cactus_detectors(instrument)
   CASE instrument OF
    'SECCHI': for d=0,n_elements(detectors)-1 do cactus_download_secchi, juldays, detector=detectors[d], sc=sc, no_logfile=no_logfile
    'LASCO' : for d=0,n_elements(detectors)-1 do cactus_download_lasco,  juldays, detector=detectors[d], no_logfile=no_logfile
   ENDCASE
 
; ------------------------closing remarks-------------------------------
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close
END
