;======================== header ===============================================
;+
; NAME:
;  cactus_header 
; PURPOSE: 
;  makes the html header
; CATEGORY:
;   cactus 
; CALLING SEQUENCE:
;   subroutine of catalo.pro
; INPUTS:
;   given_file,firstc2,latestc2,firstc3,latestc3   
;
; KEYWORD PARAMETERS:
;  
; OUTPUTS:
;   
; COMMON BLOCKS:
;  
; CALLS:
;
; MODIFICATION HISTORY:
;  ------------------ version 2.0.0 ----------------------------
;  ------------------ version 2.0.1 ----------------------------
;   2007/08/07, ER, the header of the cmecat.txt is now made by make_header.pro
;  ------------------ version 2.1.0 ----------------------------
;  ------------------ version 2.2.0 ----------------------------
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, cactus_preamble removed
;   2008/03/20, ER, cactus_preamble again introduced + adapted for secchi. First and last index is retrieved here (not anymore in catalo).
;   2008/04/15, ER, put threshold and minimal CME-width info in the header
;   2009/03/02, ER, add keyword factor
;-
;======================== end header ===========================================
pro cactus_header, filename, instrument=instrument, thresh=minpeak, width=min_w, factor=factor,no_logfile=no_logfile

   cactus_preamble, dir,myname,hostname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open
; ------------------------main code ------------------------------------

 	detectors=cactus_detectors(instrument)

	IF file_test(filename) THEN BEGIN
         printf, log, filename,'exists, header is appended'
         openw, outputfile, filename, /get_lun,/append 
	ENDIF ELSE BEGIN 
	 printf, log, filename,'does not exist, header is created'
         openw, outputfile, filename, /get_lun
	ENDELSE
	printf, outputfile, ':Issued: ', systime(/UTC)
	printf, outputfile, ':Product: CACTus catalogue (http://sidc.be/cactus)'
	printf, outputfile, '#----------------------------------------------------------------------'
	printf, outputfile, '# Instrument: '+instrument+'| Detector: '+detectors
	printf, outputfile, '# Threshold : '+strtrim(minpeak,2)+' | Factor : '+strtrim(factor,2)+' | Minimal CME width: ',strtrim(min_w,2)
        printf, log, 'Input from instrument: '+instrument
	printf, outputfile, '# '
    for d=0,n_elements(detectors)-1 do begin 
        file=dir.work+detectors[d]+'index.sav'
        IF file_test(file) THEN BEGIN
     	restore, file
	n=n_elements(index)-1
 	printf, outputfile, '  first '+detectors[d]+': '+ index[0].date_obs+'  '+index[0].time_obs+' '+index[0].filename
      	printf, outputfile, '  last  '+detectors[d]+': '+ index[n].date_obs+'  '+index[n].time_obs+' '+index[n].filename
	printf, log, '  first '+detectors[d]+': '+ index[0].date_obs+'  '+index[0].time_obs+' '+index[0].filename
    	printf, log, '  last  '+detectors[d]+': '+ index[n].date_obs+'  '+index[n].time_obs+' '+index[n].filename 
        ENDIF 
    endfor
	printf, outputfile,'# '
	printf, outputfile,'#----------------------------------------------------------------------'
	printf, outputfile,'# Output: Detected cmemap with the following characteristics:'
	printf, outputfile,'# '
	printf, outputfile,'#    CME: CME number '
	printf, outputfile,'#   Flow: Flow number. Flows are suspicious detections, their color in the detectionmap is dark blue'
	printf, outputfile,'#     t0: onset time, earliest indication of liftoff'
	printf, outputfile,'#    dt0: duration of liftoff (hours)'
	printf, outputfile,'#     pa: principal angle, counterclockwise from North (degrees)'
	printf, outputfile,'#     da: angular width (degrees), '
	printf, outputfile,'#      v: median velocity (km/s)'
	printf, outputfile,'#     dv: variation (1 sigma) of velocity over the width of the CME'
	printf, outputfile,'#  mindv: lowest velocity detected within the CME'
	printf, outputfile,'#  maxdv: highest velocity detected within the CME'
	printf, outputfile,'#  halo?: II if da>90, III if da>180, IV if da>270, indicating potential halo/partial halo CME'
	printf, outputfile,'# '
	
        close, outputfile
        free_lun, outputfile

; ------------------------closing remarks-------------------------------
 cactus_loginfo, myname, log, no_logfile=no_logfile, /close



END



