;======================== header ===============================================
;+
; NAME: cactus_infoline
;    
; PURPOSE:
;   This function reads the indicated cmecat structure line and print the infoline in txt or html format. 
;   Used in catalo.pro and out.pro
;
; OUTPUTS:
;   infoline  
;
; CALLS:
;   cactus_cme_ID
;   strnr
;
; MODIFICATION HISTORY:
;   2006/02/27, ER,  Created
;   2006/08/03, ER,  Quality removed from infoline and add # of quadrants if width > 90
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, cactus_preamble removed
;-
;======================== end header ===========================================
FUNCTION cactus_infoline,cmeindex,html=html

; ------------------------main code ------------------------------------
 CASE (FIX(cmeindex.width)-1)/90 OF 
   0: width='   '
   1: width=' II'
   2: width='III'
   3: width=' IV'
 ENDCASE  
 
 IF not keyword_set(html) THEN BEGIN
  
  infoline=  '  '+STRNR(cmeindex.nr,4)+'|'+$
 	     cmeindex.date+' '+STRMID(cmeindex.time,0,5)+'| '+ $
             STRNR(cmeindex.dt0,2)+' | '+$
             STRNR(cmeindex.angle,3)+'| '+$
             STRNR(cmeindex.width,3)+'| '+$
             STRNR(cmeindex.speed,4)+'| '+$
             STRNR(cmeindex.speedsigma,4)+'| '+$
             STRNR(cmeindex.min_speed,4)+'| '+$
             STRNR(cmeindex.max_speed,4)+'| '+$
	     width    ;+'| '  +STRTRIM(cmeindex.quality,2)		    

 ENDIF ELSE BEGIN

  ;htmldir='diffmovie/'
  ;link=' <a href="'+htmldir+cmeindex.type+STRMID(STRTRIM(cmeindex.nr+10000,2),1,4)+'.html" target="_blank" >' 

  cactus_cme_ID, cmeindex, ID,yyyy,mm,dd,ho,mi,se,thisdir
  cmedir=STRTRIM(html,2)+thisdir  
  link=' <a href="'+thisdir+'CME.html" target="_blank" >' 	

  infoline=  ' '+link+ $
 	     STRNR(cmeindex.nr,4)+'</a>|'+$
  	     cmeindex.date+' '+STRMID(cmeindex.time,0,5)+'| '+ $
             STRNR(cmeindex.dt0,2)+' | '+$
             STRNR(cmeindex.angle,3)+'| '+$
             STRNR(cmeindex.width,3)+'| '+$
             STRNR(cmeindex.speed,4)+'| '+$
             STRNR(cmeindex.speedsigma,4)+'| '+$
             STRNR(cmeindex.min_speed,4)+'| '+$
             STRNR(cmeindex.max_speed,4)+'| '+$
	     width    ;+'| '  +STRTRIM(cmeindex.quality,2)		    

 ENDELSE
 
return, infoline

END
