;======================== header ===============================================
;+
; NAME:
;   cactus_mailer
; PURPOSE:
;   send email messages
; CATEGORY:
;   cactus 
; INPUTS:
;
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
; OUTPUTS:
;    
; COMMON BLOCKS:
;   none
;
; CALLS:
;
; MODIFICATION HISTORY:
;   2004/04/01, ver 1.00, David Berghmans created 
;   2004/11/25, ER, multiple entries possible now in cactusmailer, cactusmailer only started when needed
;   2004/12/07, RAMVDL, solved problem created by previous modification (if all CMEs in mail_line were
;                       already reported, an out-of-bounds error occurred in the array called message)
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;-
;======================== end header ===========================================
PRO cactus_mailer, mail_line, time_line, no_logfile=no_logfile, message=message

   cactus_preamble, dir,myname, hostname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------

   ; sent_mailbox is a file containing information on which CME alerts have been sent already in the past
   ; the forbidden array contains the obstime of the CMEs that got an alert already. These obstimes mark
   ; time intervals (tolerance=2.4 hours (=0.1 days)) for which it is forbidden to send a new alert
   sent_mailbox=dir.work+'sent.sav'
   findfile_out=findfile(sent_mailbox)
   file_exists = findfile_out NE ''
   IF file_exists THEN restore, sent_mailbox ELSE forbidden=[0.]

  ;initialise message
   message=[' ']

  ;check CME entries
   for i=0,n_elements(mail_line)-1 do begin
      firstobstime =   time_line[i]
      infoline     =   mail_line[i]
      IF min(abs(forbidden-firstobstime)) gt 0.1 THEN BEGIN
         forbidden=[forbidden,firstobstime]
         message=[message,infoline]
         printf, log, 'New CME detected: '+infoline 
      ENDIF ELSE BEGIN
         printf, log, 'This CME was already reported: '+infoline 
      ENDELSE         
   endfor
   
   IF n_elements(message) gt 1 THEN BEGIN
      
  ;prepare email message
      messagefile=dir.work+'cactus'
      openw, outputfile, messagefile, /get_lun
      printf, outputfile,'  A halo or partial-halo CME was detected with the following characteristics:'
      printf, outputfile,'  '
      printf, outputfile,'  	t0      | dt0| pa | da |  v  |  dv | minv| maxv|'
      printf, outputfile, message[1:*]
      printf, outputfile,'  '
      printf, outputfile,'      t0: onset time, earliest indication of liftoff'
      printf, outputfile,'     dt0: duration of liftoff (hours)'
      printf, outputfile,'      pa: principal angle, counterclockwise from North (degrees)'
      printf, outputfile,'      da: angular width of the CME (degrees), '
      printf, outputfile,'       v: median velocity (km/s)'
      printf, outputfile,'      dv: variation (1 sigma) of velocity over the width of the CME'
      printf, outputfile,'   mindv: lowest velocity detected within the CME'
      printf, outputfile,'   maxdv: highest velocity detected within the CME'
      printf, outputfile,'  '
      ;;; printf, outputfile,'   !!! THESE HALO ALERTS WERE DELAYED DUE TO A TECHNICAL PROBLEM !!! '
      printf, outputfile,'  '
      printf, outputfile,'  '
      printf, outputfile,'  This message is sent whenever a CME wider than 180 degrees is detected by cactus.'
      close, outputfile

  ;copy message to titus, from where it will be sent
      machine="titus.oma.be"
      command='scp -q '+messagefile+' sidc@'+machine+':/home/sidc/rwc/pred/new/'
      spawn, command
      command='ssh sidc@'+machine+' touch /home/sidc/rwc/flagfiles/mail_sender'
      spawn, command
      printf, log, 'Message sent to titus' 
   
      machine="sol017.oma.be"
      command='scp -q '+messagefile+' sidc@'+machine+':/home/sidc/rwc/pred/new/'
      spawn, command
      command='ssh sidc@'+machine+' touch /home/sidc/rwc/flagfiles/mail_sender'
      spawn, command
      printf, log, 'Message sent to sol017' 

      save, filename=sent_mailbox, forbidden
   ENDIF
 ;--------------- closing remarks --------------------------------------- 
   cactus_loginfo, myname, log, no_logfile=no_logfile, /close


END
