;+
; NAME:
;   cactus_morf_time_cleaning
;
; PURPOSE:
;   This function performs the mathematical morfological time
;   cleaning. First the cube will be searched for clear regions and
;   points. For every clear region the cleaning decides if a clear 
;   region is a streak or not. In the case of a streak and clear
;   points the pixels will be corrected. 
;
; USE:
;   cleaned_cube = cactus_morf_time_cleaning(
;                                          polar,
;                                          index
;                                          [,high_thresh=high_thresh]
;                                          [,low_thresh=low_thresh]
;                                         ) 
;
; INPUTS:
;     parameters:
;         polar:          the cube with images in polar coördinates to
;                         clean
;
;         index:          the index corresponding with the images
;
;     keywords:
;         high_thresh:    see cactus_create_regions for more
;                         documentation
;
;         low_thresh      see cactus_create_regions for more
;                         documentation
;
; OUTPUTS:
;     The corrected cube.
;
; MODIFICATION HISTORY:
;     2010/08/12, BB, Created
;======================================================================
;-
FUNCTION cactus_morf_time_cleaning,polar,index ,high_thresh=high_thresh,low_thresh=low_thresh
  cactus_preamble, dir, my_name

  diffcube=rundiff_cube(index,polar,/ratio)
  diffcube=2.5*(diffcube<0.02)
   
  nimages  = (size(polar))[3]
  nrad = (size(polar))[2]
  nang = (size(polar))[1] 
 
  mask1 = intarr(nang,nrad,nimages)
  regions = cactus_create_regions(diffcube,high_thresh=high_thresh,low_thresh=low_thresh,mask1=mask1)
  diffcube = 0
  save, mask1,regions,filename=dir.tmp+path_sep()+'morf_time_temp.sav'
  mask1=0
  regions=0
  diffcube_nr = rundiff_cube(index,polar)
  restore,dir.tmp+path_sep()+'morf_time_temp.sav'
  file_delete,dir.tmp+path_sep()+'morf_time_temp.sav',/ALLOW_NONEXISTENT
  vegen = cactus_search_streaks_and_bright_points(regions,mask1,diffcube_nr,/dilate)
  diffcube_nr = 0
  regions = 0
  mask1=0
  RETURN, cactus_correct_polar(vegen, polar)
END
