;======================== header ===============================================
;+
; NAME: cactus_out
;    
; PURPOSE:
;   Produces and collects all files needed for the online cactus status website.
;   All these files are stored in the out directory. This program does not do 
;   the actual transfer to the website. That task is dedicated to website.pro
; CATEGORY:
;   cactus 
; INPUTS:
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;   /click_map: if set the html page will be such that detectionmap is clickable
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   cactus_cmemovie.pro
;   cactus_diffmovie.pro
;   cactus_cmemap.pro
;   cactus_click_map.pro (only if keyword_set(click_map))
;   cactus_veloplot.pro
;   cactus_cme_ID
;   cactus_cme_html
;   cactus_infoline
;
; MODIFICATION HISTORY:
;   2002/05/07, ver 1.00, David Berghmans
;   2002/06/02, DB  also producing index.html now
;   2002/08/23. DB, crontab keyword
;   2003/06/10, DB, pipe dir removed
;   2004/12/16, ER, weblink to speed-plots added     
;   2004/12/21, ER, html page per CME improved     
;   2005/03/30, ER, correct html syntax of 'latestCMEs.html'.     
;   2005/11/23, ER, enter information on version number in html file.     
;   -------------   version 2.0.0 frozen here   -----------
;   2005/11/30, ER, adapt to be compatible with the new numbering. We now have CMEs and FLOWs (<30 pixels)
;		    The CME-numbers are tracked in the consecutive detections, the FLOWs not.     
;   2006/02/16, ER, add changes to cme_html and plot detectionmap right of cmecat.txt
;   2006/02/24, ER, Read numbers from cmecat_struct instead of numbers.sav
;   2006/02/24, ER, make speed plot per cme in cme_velo.pro
;   2006/03/14, ER, only execute out and website if there are detections
;   -------------   version 2.1.0 -------------------------
;   2007/08/08, ER&SH, Make cmemap clickable => added keyword /click_map
;   -------------   version 2.2.0 -------------------------
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/01/09, ER, cactus.jpg and styles.css now also in $CACTUSPROGRAMDIR/doc
;   2008/01/15, ER, insertMov.js now also in $CACTUSPROGRAMDIR/doc (needed for playing movie directly in html)
;   2008/01/16, ER, now call cacus_cme_txt directly from out.pro
;   2008/01/16, ER, bug corrected: if ncme gt 0 then ...
;   2008/01/30, ER, replace spawns by their IDL equivalents
;   2008/03/12, ER, remove crontab keywords
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/19, ER, Quit IDL if input files do not exist
;   2008/03/21, ER, do not write cmecat.txt in  cactus_catalo.pro, do it here, add instrument keyword to cactus_out
;   2008/03/26, ER, cmemap is by default clickable now
;   2008/04/15, ER, add width=min_w as keyword to out for passing it to header.pro
;   2009/03/02, ER, add keyword factor
;   2009/05/22, ER, change keyword no_movie into movie: default is no_movie
;   2009/05/22, DB, clean-up jpeg files produced when movie making
;   2009/05/29, ER, clean-up files ONLY if /movie is set. Also replace spawn command with IDL equivalent (file_delete)
;   2009/11/09, ER, replace exit statement with GOTO, jump to avoid jumping outside IDL
;
;-
;======================== end header ===========================================
PRO cactus_out , no_logfile=no_logfile, no_click_map=no_click_map,movie=movie,thresh=thresh, width=min_w, instrument=instrument, factor=factor

    cactus_preamble, dir,myname
    cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------ keyword settings ----------------------------
    IF NOT KEYWORD_SET(thresh)  THEN thresh = 0.3      ; set default intensity threshold
    IF NOT KEYWORD_SET(min_w)   THEN min_w = 8         ; set default width 
; ------------------------main code ------------------------------------

 ;----------------------- CACTus-version info on html page -----------------------
   openr, fu, dir.var+'version.txt',/get_lun
   line=' '
   readf, fu, line
   free_lun, fu

   version=strmid(line,5,5,/reverse_offset)
   file_copy, dir.var+'cactus.jpg', dir.main,/overwrite
   file_copy, dir.var+'styles.css', dir.main,/overwrite
   file_copy, dir.var+'insertMov.js', dir.out,/overwrite
   files=[file_search(dir.out,'*.txt'),file_search(dir.out,'Flow*'),file_search(dir.out,'CME*'),file_search(dir.images+'LASCO*')]
   if total(strlen(files)) gt 0 then file_delete, files[where(strlen(files) gt 0)], /allow_nonexistent,/recursive, /quiet
 
 ; -------------------- get and read cmecat.txt from catalodir --------------------


; ------------------ sort CMEs and FLOWs + make html page per event -------------------
   IF NOT FILE_TEST(dir.work+'CMEmap.sav') THEN BEGIN
     printf, log, "  Input files do not exist. Jumping to end of program."
     GOTO, jump
   ENDIF


  restore, dir.work+'CMEmap.sav'  ;cmemap
  restore, dir.out+'cmecat_struct.sav' ; cmecat
  where_cme=where(cmecat.type eq 'CME',ncme) 
  where_flow=where(cmecat.type eq 'Flow',nflow) 

  if (ncme+nflow) gt 0 then begin
  
   if nflow gt 0 then begin
     flow=cmecat[where_flow]
     flow=flow[reverse(sort(flow.nr))]
   endif
 
   if ncme gt 0 then begin 
     cme=cmecat[where_cme] 
     cme=cme[reverse(sort(STRMID(cme.date,0,4)+STRMID(cme.date,5,2)+STRNR(cme.nr)))]   
   endif   

   for i=1,n_elements(cmecat)-1 do begin
	cactus_cme_id, cmecat[i], ID, yyyy,mm,dd,ho,mi,se,thisdir	;cactus function which extracts the ID, year, month, day, etc ...
;	yeardir=dir.out+yyyy+'/'  
;	monthdir=dir.out+mm+'/' 
	cmedir=dir.out+thisdir
	
;	if not file_test(yeardir, /directory)  then file_mkdir, yeardir
;	if not file_test(monthdir, /directory) then file_mkdir, monthdir
	if not file_test(cmedir, /directory)   then file_mkdir, cmedir
        printf, log, '          Treat  ' ,STRNR(cmecat[i].nr),' ID=',ID
	cactus_cme_html,cmecat[i],cactus_headerline(cmecat[i].type),cmedir		
    endfor
  endif
 ;----------------------- make cmecat.txt -----------------------
       textfile=dir.out+"cmecat.txt"
       cactus_header, textfile, instrument=instrument, thresh=thresh, width=min_w, no_logfile=no_logfile, factor=factor

       openw, iu, textfile, /get_lun, /append
       printf, iu, cactus_headerline('CME')
       if ncme gt 0 then for i=0, ncme-1  do printf, iu, cactus_infoline(cme[i])       
       printf, iu, ' '
       printf, iu, cactus_headerline('Flow')
       if nflow gt 0 then for i=0, nflow-1 do printf, iu, cactus_infoline(flow[i])
       close, iu
       free_lun, iu
 ;----------------------- make latestCMEs.html  -----------------------

   filename="latestCMEs.html"
   openw,iu, dir.out+filename, /get_lun  ; iu stands for indexunit
   printf, iu, '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhmtl1-transitional.dtd">'
   printf, iu, '<html xmlns="http://www.w3.org/1999/xhtml">'
   printf, iu,'<head>'
   printf, iu,'<meta http-equiv="content-type" content="text/html; charset=utf-8" />'
   printf, iu,'<title>CACTus Diagostics</title>'
   printf, iu,'<meta name="keywords" content="CACTus, CME, coronal mass ejections, LASCO, SECCHI, automated detection, space weather" />'
   printf, iu,'<meta name="description" content="" />'
   printf, iu,'<link href="../styles.css" rel="stylesheet" type="text/css" />'
   printf, iu,'<script type="text/javascript" src="insertMov.js"></script>'
   printf, iu,'</head>'
   printf, iu,'<body>'
   printf, iu,'<div id="header">'
   printf, iu,'<div id="logo">'
   printf, iu,'<a href="http://www.sidc.be/cactus"><img src="../cactus.jpg" alt="logo" class="logo"/></a>'
   printf, iu,'<h1>CACTUS<div class="ver">',version,'</div></h1>'
   printf, iu,''
   printf, iu,"<h2>A software package for 'Computer Aided CME Tracking'</h2>"
   printf, iu,'</div>'
   printf, iu, '</div>'
   printf, iu, '<div id="latestCME">'
   printf, iu, '<div style="clear: both"><hr /></div>'
   printf, iu, " <h2> CMEs detected by Cactus </h2>"
   printf, iu, "<table border=0 cellspacing=0 cellpadding=0 width=100%>"
   printf, iu, "  <tr>"
   printf, iu, "<PRE>"
   close, iu
   free_lun, iu
   cactus_header, dir.out+filename, instrument=instrument, thresh=thresh, width=min_w, no_logfile=no_logfile, factor=factor
   openw,  iu, dir.out+filename, /get_lun, /append
   printf, iu, "  </PRE>"
   printf, iu, " <td valign='top'>"
   printf, iu, "  <PRE>"
   printf, iu, cactus_headerline('cme')
   if ncme gt 0 then for i=0, ncme-1  do printf, iu, cactus_infoline(cme[i],html=dir.out)
   printf, iu, " "
   printf, iu, cactus_headerline('Flow')
   if nflow gt 0 then for i=0, nflow-1 do printf, iu, cactus_infoline(flow[i],html=dir.out)
   printf, iu, "#EOF" 
   printf, iu, "  </PRE>"
   printf, iu, " "
   printf, iu, " "
   printf, iu, " "
   printf, iu, " <a href='cmecat.txt' target='_blank'>cmecat.txt</a>"
   ;printf, iu, ' <p><script type="text/javascript">InsertMov("CACTus_movie.mov","360","471");</script></p>'
   printf, iu, " </td>" 
   ; include detection map and make it clickable unless no_click_map is set as keyword
   printf, iu, ' <td valign="top"><img src="detectionmap.png" alt="detectionmap" usemap="#detectionmap" /></td>'
   printf, iu, ' <map id ="detectionmap" name="detectionmap">'  
   IF NOT keyword_set(no_click_map) THEN BEGIN
    printf, log, '   You have asked to make the detectionmap clickable.'
    cactus_click_map, iu, cmemap, cmecat, log=log
    printf, log, '   cactus_click_map done'
   ENDIF 
   printf, iu, ' </map>'
   printf, iu, "  </tr>"
   printf, iu, "  <tr>"
   printf, iu, "    <td colspan='2'><hr></td>"
   printf, iu, "  </tr>"
   printf, iu, "</table>"
   printf, iu, "</div>"
   printf, iu, "</body>"
   printf, iu, "</HTML>"
  close, iu
  free_lun, iu

 ; log   
   printf, log, '   latestCMEs.html made'
 ;----------------------- make reldev diagnostics-------------------------
   restore, dir.work+'CMEmap.sav' 	; cmemap,myvelocity
   cactus_veloplot, cmecat,cmemap,myvelocity
   printf,log, '   cactus_veloplot done'
   
   cactus_cmemap, cmecat, no_logfile=no_logfile
   printf,log, '   cactus_cmemap done'
   
   cactus_diffmovie, no_logfile=no_logfile,movie=movie, instrument=instrument
   printf,log, '   cactus_diffmovie done'

   cmemap = 0

   IF ncme+nflow GT 0 then begin 
   cactus_cme_txt, no_logfile=no_logfile,movie=movie, instrument=instrument,cmecat
   printf,log, '   cactus_cme_txt done'
 
   IF KEYWORD_SET(movie) THEN begin 
   cactus_cmemovie, no_logfile=no_logfile, thresh=thresh
   printf,log, '   cactus_cmemovie done'
   ; remove all -if any- jpeg files in the work/images directory (these are temporary files when producing the movies)
   file_delete, dir.images+strupcase(instrument)+'-*.jpeg', /allow_nonexistent
   endif ;movie
  ENDIF

;   string=''
;   openr, file, dir.out+'cmecat.txt', /get_lun
;   readf, file, string
;   year   = STRMID(string,29,4)
;   month  = STRMID((100+(month_id(STRMID(string,13,3)))(0)),6,2)
;   day    = STRMID(100+STRMID(string,17,2),6,2)
;   hour   = STRMID(string,20,2)
;   min    = STRMID(string,23,2)
;   close, file
;   date=year+month+day+hour+min
  
   jump:
 
 ; ------------------------closing remarks-------------------------------
   cactus_loginfo, myname, log, no_logfile=no_logfile, /close
  
END
