;+
; NAME: 
;   cactus_pixels_in_range
;
; PURPOSE:
;   This function gives all the pixels gt zero within a certain
;   range described by the keywords.
;
; USE:
;     cactus_pixels_in_range, image,
;                             min_angle=min_angle,
;                             max_angle=max_angle, 
;                             min_radius=min_radius,
;                             max_radius=max_radius,
;                             pos_streak=pos_streak,  
;                           [pos=pos]
;    If pos_streak is used, min_angle, max_angle, min_radius,
;    max_radius are neglected.
;
;    Min_angle, max_angle, min_radius, max_radius musst be used
;    together
;
; INPUTS:
;     parameters:
;          image:       the image to search in
;
;     keywords:
;          min_angle:	the minimum angle of the interval
;          max_angle:	the maximum angle of the interval
;          min_radius:  the minimal radius of the interval
;          max_radius:  the maximum radius of the interval
;          pos_streak:  the position in wich neighborhood
;                       we have to search
;
; OUTPUTS:
;     returns: 
;        The number of pixels found
;
;     Keywords
;        pos:         the position of the points
;
; MODIFICATION HISTORY:
;   2010/08/12, BB, Created
;================================================================
;-
FUNCTION cactus_pixels_in_range,$
   image,$
   pos_streak=pos_streak,$
   min_angle=min_angle,$
   max_angle=max_angle,$
   min_radius=min_radius,$
   max_radius=max_radius,$
   pos=pos
  ;creation of structure element: cirkel r=10
  struct = replicate(1,21,21)
  struct[0,0:6]=0
  struct[0,14:20]=0
  struct[0:6,0]=0
  struct[14:20,0]=0
  struct[1,0:4]=0
  struct[1,16:20]=0
  struct[0:4,1]=0
  struct[16:20,1]=0
  struct[2,0:2]=0
  struct[2,18:20]=0
  struct[0:2,2]=0
  struct[18:20,2]=0
  struct[18,0:2]=0
  struct[19,0:4]=0
  struct[20,0:6]=0
  struct[18,18:20]=0
  struct[19,16:20]=0
  struct[20,14:20]=0
  struct[0:6,20]=0
  struct[0:4,19]=0
  struct[0:2,18]=0
  struct[18:20,18]=0
  struct[16:20,19]=0
  struct[14:20,20]=0
  struct[10,10]=0

  n_nr_o_p = 0

  IF keyword_set(pos_streak)THEN BEGIN
     streak = image*0
     streak[pos_streak]=1
     streak = dilate(streak,struct)
     pos_pixels = where(streak GT 0, npos_pix)
     IF npos_pix GT 0 THEN BEGIN
        nr_o_pixels = where(image[pos_pixels] GT 0,n_nr_o_p)
     ENDIF ELSE n_nr_o_p = 0
     IF keyword_set(pos)THEN BEGIN
        IF n_nr_o_p GT 0 THEN BEGIN
           positions = image*0
           positions[pos_pixels[nr_o_pixels]] = 1
           pos = where(positions GT 0)
        ENDIF ELSE pos = -1        
     ENDIF
  ENDIF ELSE BEGIN
     npixely = (size(image))[2]
     this_min_r = min_radius-10 > 0
     this_max_r = max_radius+10 < npixely-1
     pixels = image[min_angle:max_angle,this_min_r:this_max_r]
     nr_o_pixels = where(pixels GT 0, n_nr_o_p)
     IF keyword_set(pos)THEN BEGIN
        IF n_nr_o_p GT 0 THEN BEGIN
           positions = image*0
           positions[min_angle:max_angle,this_min_r:this_max_r] = pixels
           pos = where(positions GT 0)
        ENDIF ELSE pos = -1
     ENDIF
  ENDELSE
  
  RETURN, n_nr_o_p
END
