;======================== header ===============================================
;+
; NAME:
;   cactus_remetf
; PURPOSE:
;   REMove Exposure Time Fluctuation
;
; CATEGORY:
;   cactus 
;
; INPUTS:
;   
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   
; MODIFICATION HISTORY:
;   2002/05/19 DB, created
;   2003/04/24 DB, no more assuming npixel1=360
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/19, ER, Quit IDL if input files do not exist
;   2008/03/20, ER, replace 'telescope' with 'detector'; also remove 'Sanity check'
;   2008/03/26, ER, take out all restore and save files. 
;   2009/04/14, ER, save the timeline (used in cactus_diffmovie.pro) and take out the xycube operations
;-
;======================== end header ===========================================
PRO cactus_remetf, index,polar,no_logfile=no_logfile

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

 ; ------------------------main code ------------------------------------

   sz=size(polar)
   npixel1=sz[1]
   npixel2=sz[2]
   nimages=sz[3]
  
 ; determine the good locations (not in missing blocks)
   good = polar GT 0
   s=replicate(1,3,3)
   for i=0,nimages-1 do good[*,*,i]=erode(erode(good[*,*,i],s),s)
   ngood=total(good,3)>1

 ; derive statistics to find out which locations are the least variable
   avim=total(polar*good,3)/ngood
   stdev = avim*0 ; to get dimensions right
   for i=0, nimages -1 do stdev=stdev + good[*,*,i]*(polar[*,*,i]-avim)^2
   stdev=sqrt(stdev/ngood)
   relstdev=stdev/(avim>1)
   good_enough=where(ngood gt  max(ngood)-5)
   medrelstdev=median(relstdev[good_enough])
   printf, log, 'Statistics of datacube calculated.'
  
 ; determine quiet locations, away from the Sun and occultor
   mask=stdev*0. & mask[FIX(0.1*npixel1):FIX(0.9*npixel1),*]=1.
   quiet= relstdev LT medrelstdev and (mask) and (ngood gt max(ngood)-5)
   printf, log, 'Fraction of the image selected as quiet area:', $
         total(quiet)/total(quiet GE 0)

  ; make timeline of quiet areas only
   timeline=FLTARR(nimages)
   for i=0, nimages -1 do good[*,*,i]=good[*,*,i]*quiet
   for i=0, nimages -1 do timeline[i]=total(polar[*,*,i]*good[*,*,i]/(avim>1))
   timeline=timeline/(total(total(good,1),1)>1)

 ; scale to 1
   av=(moment(timeline))(0)
   timeline=timeline/av
   printf, log, 'Correction factors:'
   printf, log, 'Img nr     correction factor'
   for i=0, nimages-1 do printf, log, i, timeline[i]

 ; scale
   for i=0, nimages-1 do polar[*,*,i]=polar[*,*,i]/timeline[i] 
 
   save, filename=dir.work+index[0].detector+'_remetf.sav',timeline 

 ;;;; also apply exposure time correction to xy images  
 ;;;  for i=0, nimages-1 do xycube[*,*,i]=xycube[*,*,i]/timeline[i] 

 ; ------------------------closing remarks-------------------------------
   cactus_loginfo, myname, log, no_logfile=no_logfile, /close
  
END
