;+
; NAME:  
;   cactus_search_streaks_and_bright_points
;
; PURPOSE:
;   This function filters the streaks from the non-streaks. 
;
; USE:
;   streaks_bright = cactus_search_streaks_And_bright_point(
;                                    regions,
;                                    mask1,
;                                    polar,
;                                    [/dilate]) 
;
; INPUTS:
;   parameters:
;         regions:     A cube with regions. The backgroundpixels have
;                      a value 0, the other pixels a value depending
;                      region they belong to.
;
;         mask1:       A cube with the thresholded points. Must be 
;                      retrieved from cactus_create_regions.
;
;         polar:       A cube with the original polar images.
;
;   keywords:
;         dilate:      Use this keyword if dilation must be used
;
; OUTPUTS:
;   returns:
;     A cube with the same dimension as regions cube with 1 on the
;     places where a streak was found.
;
; MODIFICATION HISTORY:
;   2010/08/12, BB, Created
;
;-
FUNCTION cactus_search_streaks_and_bright_points,regions,mask1,polar,dilate=dilate
  nimages = (size(regions))[3]
  nrad = (size(regions))[2]
  nang = (size(regions))[1]
  structure = bytarr(9,9)+1
  structure[[0,1,2,3,5,6,7,8,9,10,16,17,18,26,27,35,40,45,53,54,62,63,64,70,71,72,73,74,75,77,78,79,80]]=0
  ;Checking the found regions. When in the previous and following images
  ;no regions on the same angle with different radius are found we 
  ;got a veeg 
  cl_regions = cactus_classify_region(regions)
  vegen = intarr(nang,nrad,nimages)
  IF keyword_set(dilate) THEN BEGIN
     poss_clr_p = where(dilate(regions,structure) eq 0, nposs_clr_p)
  ENDIF ELSE BEGIN
     poss_clr_p = where(regions eq 0, nposs_clr_p)
  ENDELSE
  IF nposs_clr_p GT 0 THEN BEGIN
     vegen[poss_clr_p] = mask1[poss_clr_p]
     ;vegen
     FOR imind = 0, nimages-1 DO BEGIN
        im = vegen[*,*,imind]
        reg = label_region(im,/ALL_NEIGHBORS)
        nreg = max(reg)
        FOR regn = 1, nreg DO BEGIN
           reg_pos = where(reg eq regn, nreg_pos)
           IF nreg_pos GE 5 THEN BEGIN
              reg[reg_pos]=0
           ENDIF           
        ENDFOR
        vegen[*,*,imind] = reg
     ENDFOR
  ENDIF

  mask1=0
  FOR imind = 0, nimages-1 DO BEGIN
     regions_on_im = WHERE(cl_regions.imagenumber EQ imind,nregions)
     IF nregions GT 0 THEN BEGIN        
        ;print, 'image : '+ string(imind) +'nregions : ' + string(nregions)
        FOR regionsi = 0, nregions-1 DO BEGIN
           regionstr = cl_regions[regions_on_im[regionsi]]
           region_pos = WHERE(regions[*,*,imind] EQ regionstr.regionnumber)
           ;most occuring situation not first or last image
           IF (imind GT 0) AND (imind LT nimages-1)THEN BEGIN 
              neighbours = WHERE((cl_regions.imagenumber EQ imind-1)OR(cl_regions.imagenumber EQ imind+1), nn)
           ENDIF ELSE BEGIN 
              ;case first image
              IF imind EQ 0 THEN BEGIN
                 neighbours = WHERE(cl_regions.imagenumber EQ imind+1, nn)
              ENDIF ELSE BEGIN
                 ;case last image
                 IF imind EQ nimages-1 THEN BEGIN
                    neighbours = WHERE(cl_regions.imagenumber EQ imind-1, nn)
                 ENDIF ELSE BEGIN
                    ;impossible case
                    MESSAGE,'WHOOHOO you did the impossible'
                 ENDELSE
              ENDELSE
           ENDELSE
           ;check if there are neighbours with regions
           ;yes continu checking 
           ;no veeg
           got_veeg = 0
           IF nn GT 0 THEN BEGIN
              ;checking if within the neighbours there are region 
              ;in the same angle. No => veeg, yes extra checking
              neig_struct = cl_regions[neighbours]
              min_anglelimit = regionstr.minangle - 5>0
              max_anglelimit = regionstr.maxangle + 5<179
              n_poss_struct = WHERE((neig_struct.minangle GT max_anglelimit)$
                                    OR $ 
                                    (neig_struct.maxangle LT min_anglelimit),$
                                    n_ngang,$
                                    complement = poss_struct,$
                                    ncomplement = ngang $
                                   )
              IF ngang GT 0 THEN BEGIN
                 ;now we check if the found regions lie
                 ;near the region
                 same_angle_str = neig_struct[poss_struct]
                 min_a = 0
                 min_b = 0
                 min_global = 0
                 IF imind GT 0 AND imind LT nimages-1 THEN BEGIN
                    pos_b = where(same_angle_str.imagenumber EQ imind-1,n_pos_b)
                    pos_a = where(same_angle_str.imagenumber EQ imind+1,n_pos_a)
                    IF n_pos_a GT 0 THEN BEGIN
                       same_ang_str_a = same_angle_str[pos_a]
                       min_a = min(same_ang_str_a.minradius - regionstr.maxradius)
                    ENDIF
                    IF n_pos_b GT 0 THEN BEGIN
                       same_ang_str_b = same_angle_str[pos_b]
                       min_b = min(regionstr.minradius-same_ang_str_b.maxradius)
                    ENDIF
                    IF n_pos_a GT 0 AND n_pos_b GT 0 THEN BEGIN 
                       min_global = min([min_a,min_b]) 
                    ENDIF ELSE BEGIN
                       IF n_pos_a EQ 0 THEN min_global = min_b ELSE min_global = min_a
                    ENDELSE                       
                 ENDIF ELSE BEGIN
                    IF imind EQ 0 THEN BEGIN
                       pos_a = where(same_angle_str.imagenumber EQ imind+1,n_pos_a)
                       IF n_pos_a GT 0 THEN BEGIN
                          same_ang_str_a = same_angle_str[pos_a]
                          min_a = min(same_ang_str_a.maxradius - regionstr.minradius)
                       ENDIF
                       min_global = min_a
                    ENDIF ELSE BEGIN
                       pos_b = where(same_angle_str.imagenumber EQ imind-1,n_pos_b)
                       IF n_pos_b GT 0 THEN BEGIN
                          same_ang_str_b = same_angle_str[pos_b]
                          min_b = min(regionstr.minradius-same_ang_str_b.maxradius)
                       ENDIF
                       min_global = min_b
                    ENDELSE
                 ENDELSE
                 
                 IF min_global GT 5 THEN BEGIN
                    ;when 70% of the found regios are to far we have streak
                    ; we now have 'streak'
                    got_veeg=1
                 ENDIF ELSE BEGIN
                    ;we consider the number of pixels at the angle range 
                    ;of the region. When number of pixels within the range
                    ;in the previous and following image is less than 10%
                    ;of the checked region we consider the checked region
                    ;as a streak                    
                    IF imind GT 0 AND imind LT nimages-1 THEN BEGIN
                       ;normal case
                       number_pix_b = cactus_pixels_in_range(regions[*,*,imind-1],$
                                                           pos_streak = region_pos $
                                                          )
                       number_pix_a = cactus_pixels_in_range(regions[*,*,imind+1], $
                                                           pos_streak = region_pos $                                      
                                                          )
                       IF float(number_pix_a)/float(regionstr.numberofpixels) LT 0.2 AND $
                          float(number_pix_b)/float(regionstr.numberofpixels) LT 0.2 THEN BEGIN
                          got_veeg = 1
                       ENDIF
                    ENDIF ELSE BEGIN
                       IF imind EQ 0 THEN BEGIN
                          ;case first image
                          number_pix_a = cactus_pixels_in_range(regions[*,*,imind+1],$
                                                              pos_streak = region_pos $
                                                             )
                       IF float(number_pix_a)/float(regionstr.numberofpixels) LT 0.2 THEN BEGIN
                          got_veeg = 1
                       ENDIF
                       ENDIF ELSE BEGIN
                          ;case last image                          
                          number_pix_b = cactus_pixels_in_range($ ;regionstr.minangle,$
                                         $                      ;regionstr.maxangle,$
                                         $                      ;regionstr.minangle,$
                                         $                      ;regionstr.maxangle,$
                                         regions[*,*,imind-1],$
                                         pos_streak = region_pos $
                                                             )
                          IF float(number_pix_b)/float(regionstr.numberofpixels) LT 0.3 THEN BEGIN
                             got_veeg = 1
                          ENDIF
                       ENDELSE
                    ENDELSE                    
                 ENDELSE
              ENDIF ELSE BEGIN
                 ;no neigbours with regions in the angles space
                 got_veeg = 1
              ENDELSE
           ENDIF ELSE BEGIN
              ;no images so we can accept the case of a veeg
              got_veeg = 1              
           ENDELSE
           IF got_veeg EQ 1 THEN BEGIN
              cl_regions[regions_on_im[regionsi]].imagenumber = -1
              veeg = INTARR(nang,nrad)
              ;check why the streak is there:
              polimage = polar[*,*,imind]
              IF (imind GT 1) AND (imind LT nimages-2) THEN BEGIN
                                ;most frequent case
                 prev_prev_pol = polar[*,*,imind-2]
                 prev_pol = polar[*,*,imind-1]
                 fol_pol = polar[*,*,imind+1]
                 fol_fol_pol = polar[*,*,imind+2]
              ENDIF ELSE BEGIN
                 IF imind EQ 0 THEN BEGIN
                    prev_pol = polar[*,*,imind+1]
                    prev_prev_pol = polar[*,*,imind+2]
                    fol_pol = polar[*,*,imind+1]
                    fol_fol_pol = polar[*,*,imind+2]
                 ENDIF ELSE BEGIN
                    IF imind EQ nimages-1 THEN BEGIN
                       prev_pol = polar[*,*,imind-1]
                       prev_prev_pol = polar[*,*,imind-2]
                       fol_pol = polar[*,*,imind-1]
                       fol_fol_pol = polar[*,*,imind-2]
                    ENDIF ELSE BEGIN
                       IF imind EQ 1 THEN BEGIN
                          prev_prev_pol = polar[*,*,imind-1]
                          prev_pol = polar[*,*,imind-1]
                          fol_pol = polar[*,*,imind+1]
                          fol_fol_pol = polar[*,*,imind+2]
                       ENDIF ELSE BEGIN
                          IF imind EQ nimages-2 THEN BEGIN
                             prev_prev_pol = polar[*,*,imind-2]
                             prev_pol = polar[*,*,imind-1]
                             fol_pol = polar[*,*,imind+1]
                             fol_fol_pol = polar[*,*,imind+1]
                          ENDIF ELSE BEGIN
                             MESSAGE, 'WHOOHOO you did the impossible'
                          ENDELSE
                       ENDELSE                  
                    ENDELSE
                 ENDELSE
              ENDELSE
                                
              IF (imind GT 1) AND (imind LT nimages-1) THEN BEGIN
                 prev_less = where(prev_pol[region_pos] lt 0,nprev_less,complement=fol_less,ncomplement=nfol_less)
                 ratio = float(nprev_less)/float(nprev_less+nfol_less)
                 IF ratio GT 0.9 THEN BEGIN
                    veeg[region_pos]=1
                    vegen[*,*,imind-1]+=veeg
                 ENDIF ELSE BEGIN
                    veeg[region_pos]=1
                    vegen[*,*,imind]+=veeg
                 ENDELSE          
              ENDIF ELSE BEGIN
                 veeg[region_pos]=1
                 vegen[*,*,imind]+=veeg
              ENDELSE
           ENDIF
        ENDFOR
     ENDIF
  ENDFOR
  
  RETURN, vegen
END
