;======================== header ===============================================
;+
; NAME:
;   cactus_settings 
; PURPOSE:
;   Defines system variable !cactus, which contains directory structure
;   Must be ran prior to every use of cactus_*.pro program
; 
; CATEGORY:
;   cactus 
;
; INPUTS:
;   
; 
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;
; TO DO:
;
;
; MODIFICATION HISTORY:
;   2007/12/10, ER, Created
;   2008/01/08, ER, make directories if they don't exist. This permits to do several cactus runs simultaneously
;   2008/01/09, ER, maindir=maindir added
;   2008/01/09, ER, var moved to $CACTUSPROGRAMDIR/doc/ according to ssw standard
;   2008/01/10, ER, define a dataqkl and a datalz directory 
;   2008/01/30, ER&BN, copy and compile c-routines in the dir.dlm directory (inside work/)
;   2008/01/30, ER, replace spawns by their IDL equivalent
;   2008/02/07, ER&BN, copy and compile c-routines in the dir.dlm directory (the_total) 
;   2008/03/10, ER, change the data-directories to make it more generic and add secchi
;   2008/03/11, ER, add /secchi and /qkl as keywords
;   2008/03/12, ER, make a proper header for this routine
;   2008/03/14, ER, define a !cactusfov system variable which contains the 'absolute' cactus definitions
;   2008/03/18, ER, define the directory structure depending on the instrument (in/ and archive/)
;   2008/03/19, ER, use standard SSW envirionment variables for data-directories
;   2008/04/02, ER, add temporaty directory to dir.work, can be used as dir.tmp
;   2008/05/12, ER, programdir=expand_path(getenv('SSW_CACTUS'))
;   2008/11/07, ER, do not make directories for the first three tags, they are not accessible to the user
;	2009/05/22, DAB, 'bin' directory is now called 'src'
;- 
; =======================================================================================
PRO cactus_settings, secchi=secchi,qkl=qkl,maindir=maindir

stdum={cactus,bin:'',var:'',data:'',main:'',in:'',log:'',work:'',dlm:'',tmp:'',out:'',images:'',archive:''}

IF NOT KEYWORD_SET(maindir) THEN maindir =getenv('CACTUSDIR')
maindir=expand_path(maindir)

programdir=expand_path(getenv('SSW_CACTUS'))
stdum.bin	=programdir+path_sep()+'src'+path_sep()  ; 'src' used to be 'bin' 
stdum.var	=programdir+path_sep()+'doc'+path_sep()

stdum.main      =maindir+path_sep()
stdum.in	=maindir+path_sep()+'in'+path_sep()
stdum.work	=maindir+path_sep()+'work'+path_sep()
stdum.dlm	=stdum.work+'dlm'+path_sep()
stdum.archive	=stdum.work+'archive'+path_sep()
stdum.tmp	=stdum.work+'tmp'+path_sep()
stdum.log	=maindir+path_sep()+'log'+path_sep()
stdum.out	=maindir+path_sep()+'out'+path_sep()
stdum.images	=stdum.work+'images'+path_sep()

;define data-directories in the !cactus IDL system variable
;prepare error message when wrong combination given or instrument omitted

IF KEYWORD_SET(secchi) AND KEYWORD_SET(QKL) THEN BEGIN 
 print, '   CAVEAT!'
 print, '   Keyword qkl is ignored when using secchi data'
ENDIF

IF NOT (KEYWORD_SET (qkl) OR KEYWORD_SET(secchi)) THEN stdum.data=expand_path(getenv('LZ_IMG'))+path_sep()+'level_05'+path_sep()
IF KEYWORD_SET(qkl) and NOT KEYWORD_SET(secchi)   THEN stdum.data=expand_path(getenv('QL_IMG'))+path_sep()+'level_05'+path_sep() 
IF KEYWORD_SET(secchi)  THEN stdum.data=expand_path(getenv('secchi'))+path_sep()+'lz'+path_sep()+'L0'+path_sep()

IF NOT KEYWORD_SET(secchi)  THEN BEGIN
   if not keyword_set(qkl) then stdum.data=expand_path(getenv('LZ_IMG'))+path_sep()+'level_05'+path_sep() else $
				stdum.data=expand_path(getenv('QL_IMG'))+path_sep()+'level_05'+path_sep()
ENDIF ELSE BEGIN
   if not keyword_set(qkl) then stdum.data=expand_path(getenv('secchi'))+path_sep()+'lz'+path_sep()+'L0'+path_sep()
ENDELSE



; Define instrument dependant directories
 IF NOT KEYWORD_SET(secchi) THEN detectors=cactus_detectors('LASCO') ELSE detectors=cactus_detectors('SECCHI')
 FOR d=0,n_elements(detectors)-1 DO $ 
  stdum=create_struct(stdum,'in'+detectors[d],stdum.in+detectors[d]+path_sep(), $ 
                           'arch'+detectors[d],stdum.archive+detectors[d]+path_sep())

defsysv,'!cactus',stdum

dir=!cactus
;create directory structure
;do not make directories for the first three tags, they are not accessible to the user
FOR i=3,n_tags(dir)-1 DO $
  if NOT file_test(dir.(i),/directory) then file_mkdir, dir.(i)


; add c-code cactus_total.dlm to library
  file_copy, dir.bin+'cactus_total.dlm', dir.dlm, /overwrite
  make_dll, 'cactus_total', 'IDL_Load', EXTRA_CFLAGS='-O2', input_directory=dir.bin, output_directory=dir.dlm
  dlm_register, dir.dlm+'cactus_total.dlm'

  file_copy, dir.bin+'cactus_polar.dlm', dir.dlm, /overwrite
  make_dll, 'cactus_polar', 'IDL_Load', EXTRA_CFLAGS='-O2', input_directory=dir.bin, output_directory=dir.dlm
  dlm_register, dir.dlm+'cactus_polar.dlm'


; define the cmemap parameters-structure ------------------------------------------

cactusfov=create_struct( $
	'nangles',	180, $
	'nradii',	225,$
	'edge',		235,$
 	'km_per_pix',	float(100000),$
	'npixelx',	256,$
	'npixely',	256)

defsysv, '!cactusfov',cactusfov




END
