;======================== header ===============================================
;+
; NAME:
;   cactus_starttime (old name = startingtime)
; PURPOSE:
;   to be used in catalo.pro to calculate the real startingtimes
; CATEGORY:
;   cactus 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
; MODIFICATION HISTORY:
;   2004/03/18, ER, created 
;   2004/12/10, ER, bug corrected s.t. k+1 cannot be greater than  n_elements(original_c2)
;   2004/12/15, ER, background fit improved
;   2004/12/21, ER, movie_index added for movie per CME made in out.bin/diffmovie.pro
;   2005/01/11, ER, in 'find signal above 4*sigma: take the maximum as starting point
;   2005/04/22, ER, change calculating of c2 interval [t1,t2] + bug corrected in searching forward loop
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/03/12, ER, remove crontab keyword, always use Z-buffer for images
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/21, ER, adapted for secchi, only lines 54:59
;   2008/03/24, ER, replace telescope with instrument and source with detector
;   2008/04/08, ER, repaired bug in integration over radius, and added temporary to save memory
;   2008/04/10, ER, endtimes of cmes are saved in end_index now
;   2008/05/18, ER, make changes for SECCHI compatible with LASCO
;   2008/05/22, ER, restore datacube instead of slices (not saved anymore to save disk space) 
;   2010/07/14, BB, small changes to get the size wright 
;-
;======================== end header ===========================================
PRO cactus_starttime,no_logfile=no_logfile, instrument=instrument

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------

 ; remove previous images

   spawn, 'rm -f '+dir.images+'*.png'

 ; input
 ; note: cmes is 360 degrees wide and is non-edgehit+edgehit map (see cactus_catalo.pro)
   restore, dir.work+'datacube.sav'
   slices=transpose(temporary(datacube), [2,1,0])
   restore, dir.work+'CMEmap.sav' ; cmemap, myvelocity, first, last,tangle,cmes,pada
   restore, dir.work+'index.sav'	;index

   sz=size(cmemap)
   nangles=sz[1]
   nimages=sz[2]
  ncmemap=max(cmes)

 ; determine graphics device
   printf, log, '   Working in batchmode, using Z-buffer'
   set_plot, 'Z'
   device, set_resolution = [800,400]   ; size of image
   device, z_buffering    = 0           ; 0 for 2D

   detector=(cactus_detectors(instrument))[0]
   c2=where(strlowcase(index.detector) eq detector,complement=not_c2,ncomplement=nnot_c2)

 ; integrate over radius => 2D: set all non-c2 data to 0
   if nnot_c2 gt 0 then slices[not_c2,*,*]=0
   angle_time_plane=transpose(total(temporary(slices),2))  

 ; loop over cmes found in cmemap
   start_index=fltarr(ncmemap+1) &  start_index[*]=-1
   end_index=fltarr(ncmemap+1) &  end_index[*]=-1
   movie_index=fltarr(2,ncmemap+1)
   movie_index[*,*]=-1
   comment=strarr(ncmemap+1)


   for i=1,ncmemap do begin    ;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ FOR
    	    	    	    ;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

;calculate x(theta) and y(time) position of all elements of this CME
 pos=array_indices(cmes,where(cmes eq i,npos))
 angle=pos[0,*] & time=pos[1,*]
 left=min(angle)  & right=max(angle)


;integrate over angle => 1D
 if (left lt 180)  and (right gt 180) then begin 
  part1=total(angle_time_plane[left mod nangles:nangles-1,*],1)
  part2=total(angle_time_plane[0:right mod nangles,*],1)
  time_line=part1+part2
 endif else time_line=total(angle_time_plane[left mod 180:right mod 180,*],1)

;calculate c2 search interval 
 t1=min(time) & t2=max(time)
 IF t2 lt nimages-2 THEN t2=max(time)+2
 m1=min((c2-t1)^2,c2t1) & m2=min((c2-t2)^2,c2t2)

 movie_index[0,i]=c2t1	;first image in movie
 movie_index[1,i]=c2t2	;last image in movie

 IF (m1 gt 6) OR (m2 gt 20) OR (c2t1 EQ c2t2) THEN BEGIN  ;6 images = +/- 1h30min
 	start_index[i]=t1
	comment[i]='No C2 data'
     ENDIF ELSE BEGIN ;+++++++++++++++++++++++++++++++++++++++++++++++++ ELSE
 ;search correct starting time in c2 seach interval which is defined as [t1,t2]
 t1=c2[c2t1] & t2=c2[c2t2]
 
 ;create large window around search interval for calculating local background, from this point we consider c2 data
 shift1=30 & shift2=30
 if t1 lt shift1 then shift1 = t1
 if t2 gt nimages-1-shift2 then shift2 = nimages-t2-1

 ;x-coordinate over large window
 x=findgen(t2-t1+shift1+shift2)+t1-shift1
 c2part=where(strlowcase(index[x].detector) eq detector,nc2)
 x_c2=x[c2part]
 n=n_elements(x_c2)

 original=time_line[t1-shift1:t2+shift2]
 original_c2=original[c2part]
 bkgd=original_c2

 plot, x_c2,original_c2,color=0,background=255,$
 title='cme nr '+STRTRIM(i,2),xtitle='time',ytitle='total intensity',$
 charsize=1.5

;start iteration to calculate background "bkgd" and sigma
 line=linfit(x_c2,bkgd)
 fit=line[0]+line[1]*x_c2
 sigma=sqrt(total((bkgd-fit)^2)/n)
 ;oplot, x_c2,fit,color=0
 ;oplot, x_c2,fit+sigma,color=0,linestyle=2
 rest=where(bkgd gt fit+sigma or bkgd lt fit-2*sigma,npoints)
 iteration=0
 while npoints gt 0 do begin  ; av1 = old average & av = new average
   bkgd[rest]=fit[rest]
   line=linfit(x_c2,bkgd)
   fit=line[0]+line[1]*x_c2
   bkgd[rest]=fit[rest]
   onver=where(bkgd eq original_c2, nonver)
   sigma=sqrt(total((bkgd-fit)^2)/nonver)
  ;print, nonver
  ;oplot, x_c2,fit,color=0
  ;oplot, x_c2,fit+sigma,color=0,linestyle=2
   rest=where(bkgd gt fit+sigma,npoints)
  ;print,npoints
   iteration=iteration+1
 endwhile

 ; go one step further to improve the background fit 
 all_rest=where(original_c2 gt fit+sigma,npoints)
 bkgd[all_rest]=fit[all_rest]
 onver=where(bkgd eq original_c2, nonver)
 sigma=sqrt(total((bkgd-fit)^2)/nonver) 
 fit=smooth(bkgd,6,/edge_truncate)



 ;define the signal
 signal=time_line[t1:t2]
 xs=findgen(t2-t1+1)+t1
 c2part_s=where(strlowcase(index[xs].detector) eq detector,nsc2)
 xs_c2=xs[c2part_s]
 signal_c2=signal[c2part_s]
 oplot,xs_c2,signal_c2,color=0,thick=2

;find signal above 4*sigma
 if iteration ne 0 then begin         ;IF 2 ************************************************ IF 2
   oplot, x_c2,fit,color=0
   oplot, x_c2,fit+1*sigma,color=0,linestyle=2
   oplot, x_c2,fit+2*sigma,color=0,linestyle=2
   oplot, x_c2,fit+4*sigma,color=0,linestyle=2

   select=where(x_c2 ge t1 and x_c2 le t2)
   a=where(signal_c2 ge fit[select]+4*sigma,na)	;signal_c2=original_c2[select-1]
   IF min(a) ge 0 THEN top=max(signal_c2[a],amax) ELSE amax= 0
  
 
   IF a[amax] ge 0 THEN BEGIN         ;IF 3 **************************************** IF 3
          start=a[amax]+select[0]
          k=start
          step1=0
          oplot,[x_c2[k],x_c2[k]],[original_c2[k],original_c2[k]],psym=2,color=0
          current=original_c2[start]
          
	  if a[amax] ne 0 then begin  ;IF 4a ***************************** 
                                ;walk back to starting point*
              previous=original_c2[start-1] 	    	    	    	;*
              while (previous ge fit[k]+2*sigma) and (k gt 1) do begin	;*
               k=k-1	    	    	    	    	    	    	;*
               step1=step1+1	    	    	    	    	    	;*    
              ;print, 'k=',k	    	    	    	    	    	;*
               current=original_c2[k]	    	    	    	    	;*    
               previous=original_c2[k-1]    	    	    	    	;*
              ;print,previous,fit[k]+2*sigma 	    	    	    	;*    
               oplot,[x_c2[k],x_c2[k]],[original_c2[k],original_c2[k]]$ ;*
	            	    	    	    	    	,psym=2,color=0 ;*
              endwhile	    	    	    	    	    	    	;*
    	    	    	    	    	    	    	    	    	;*
             ;now step back between 1*sigma and 2*sigma     	    	;*
              while (previous lt current) $ 	    	    	    	;*
	      	   and (previous ge fit[k]+sigma) and (k gt 1)do begin	;*
               k=k-1	    	    	    	    	    	    	;*
               step1=step1+1	    	    	    	    	    	;*
               current=original_c2[k]	    	    	    	    	;*
               previous=original_c2[k-1]    	    	    	    	;*
               ;print,previous,current,fit[k]+sigma 	    	    	;*
               oplot,[x_c2[k],x_c2[k]],[original_c2[k],original_c2[k]]$ ;*
	            	    	    	    	    	,psym=2,color=0 ;*
              endwhile	    	    	    	    	    	    	;*
          endif ;a[amax] ne 0         ;ENDIF 4a **************************
	  
          start_index[i]=x_c2[k] & movie_index[0,i]=where(c2 eq x_c2[k])
          starttime=index[x_c2[k]].time_obs
          startdate=index[x_c2[k]].date_obs

          current=original_c2[start]
          k=start
          if start+2 lt n_elements(original_c2) then begin
          next=original_c2[start+1]
          step2=0
          ;walk further from starting point
             while (next ge fit[k]+2*sigma) and (k+2 lt n_elements(original_c2)) do begin
               k=k+1
               step2=step2+1
               ;print, 'k=',k
               current=original_c2[k]
               next=original_c2[k+1]
               ;print,next,fit[k]+2*sigma 
               oplot,[x_c2[k],x_c2[k]],[original_c2[k],original_c2[k]],psym=2,color=0
             endwhile

             ;now step back between 1*sigma and 2*sigma
             while (next lt current) and (next ge fit[k]+sigma) and (k+2 lt n_elements(original_c2)) do begin
               k=k+1
               step2=step2+1
               current=original_c2[k]
               next=original_c2[k+1]
               ;print,next,current,fit[k]+sigma
               oplot,[x_c2[k],x_c2[k]],[original_c2[k],original_c2[k]],psym=2,color=0
             endwhile
	  endif   
          end_index[i]=x_c2[k] & movie_index[1,i]=where(c2 eq x_c2[k])
          endtime=index[x_c2[k]].time_obs
          enddate=index[x_c2[k]].date_obs

          nsignal=1+step1+step2
          ;are there enough points above 2*sigma?

          if nsignal gt 0 then begin ;IF 4b **************************************
            xyouts, 100,300,'startdate = '+startdate,color=0,/device	    	;*
            xyouts, 100,280,'starttime = '+starttime,color=0,/device	    	;*
            xyouts, 100,260,'endtime  = '+endtime,color=0,/device   	    	;*
            xyouts, 100,240,'enddate  = '+enddate,color=0,/device   	    	;*
            if nsignal lt 2 then comment[i]='only 1 point above average+sigma'	;*
          endif else comment[i]='marginal case'     	    	    	    	;*
	    	    	    	     ;ENDIF 4b ***********************************
	  
	  
   ENDIF ELSE BEGIN  ;ENDIF 3 ******************************* ENFIF 3
   comment[i]='marginal case'  
   oplot, x_c2,fit,color=0
   oplot, x_c2,fit+sigma,color=0,linestyle=2
   ENDELSE
    
 endif else begin ;ENDIF 2 ************************************************************** ENDIF 2
   comment[i]='marginal case'
   oplot, x_c2,fit,color=0
   oplot, x_c2,fit+sigma,color=0,linestyle=2
 endelse

 write_png,dir.images+'intensityplot_'+STRTRIM(i,2)+'.png',tvrd()
 ENDELSE ;****************************************************************************************** ENDELSE

endfor  ;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ ENDFOR
    	;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

 ; save files    
save,filename=dir.work+'startingtime.sav',start_index,end_index,comment
save,filename=dir.work+detector+'_movie.sav',movie_index

 ; ------------------------closing remarks-------------------------------
   
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END
