;======================== header ===============================================
;+
; NAME:
;   cactus_translate
; PURPOSE:
;   to be used in catalo.pro to correct tvanglecube for the time
; CATEGORY:
;   cactus ver 4.00 (paperA&A-version, May 2004)
; MODIFICATION HISTORY:
;   2003/12/17, ER, created 
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;-
;======================== end header ===========================================
PRO cactus_translate, no_logfile=no_logfile

  cactus_preamble, dir,myname
  cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------
 ; input
     restore, dir.work+'tvanglecube.sav'   ;tvanglecube
     restore, dir.work+'index.sav'	;index
 
 ;check if the translation has already been done
 s=size(tvanglecube_trans,/type)
 IF s eq 0 THEN BEGIN
 printf, log, ' tvanglecube_trans does not exist ! '     
 
 ; start the translation    
     sz=size(tvanglecube)
     nimages   = sz[1]
     nspeeds   = sz[2]
     nangles   = sz[3]
     nradii=max(index.maxradius)+1 ;+1 to have the same size as datacube
     ref_radius=min(index.minradius); ref_radius is number of black pixels at the
	     	    	    	    ; the bottom of each slice (=15)
     tvanglecube_trans=fltarr(nimages,nspeeds,nangles)

 ; loop over angles 
     FOR angle=0,nangles-1 DO BEGIN
      tvcube=tvanglecube[*,*,angle]
      tvcube_trans=tvanglecube_trans[*,*,angle]
      ; search for datapoints
      points=where(tvcube gt 0,npoints)
      ;do a translation for each datapoint
      for i=0,npoints-1 do begin
       DT=points/nimages    	;y-coord
       t=points-DT*nimages  	;x-coord
       tvcube_trans[t+(DT*ref_radius)/nradii,DT]=tvcube[t,DT]
      endfor
     tvanglecube_trans[*,*,angle]=tvcube_trans 
     ENDFOR
     
 ; save files  
     save, filename=dir.work+'tvanglecube.sav', tvanglecube_trans,tvanglecube
 ENDIF ELSE printf, log, ' tvanglecube_trans already exists ! ' 
     
; ------------------------closing remarks-------------------------------
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END
