;======================== header ===============================================
;+
; NAME: cactus_veloplot
;    
; PURPOSE:
;   makes a plot of the velocity profile
; CATEGORY:
;   cactus 
; INPUTS:
; 
; KEYWORD PARAMETERS:
;   /crontab: set this keyword when working in the background. In this case,
;             the 'Z' buffer is use for plotting, as no real screen is
;             available.
; CALLS:
;   cactus_cme_ID
;
; COMMON BLOCKS: 
;   none
;
; MODIFICATION HISTORY:
;   2004/10/11, ER, created
;   2004/12/17, ER, moved to out.bin + write images in out/cme00##/
;   -------------   version 2.0.0 frozen here   -----------
;   2005/11/30, ER, made compatible with the number tracking + improve map-outlook
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/03/12, ER, remove crontab keyword, always use Z-buffer for images
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;-
;======================== end header ===========================================
PRO cactus_veloplot,cmecat,cmemap,myvelocity

   cactus_preamble, dir,myname
 
; ------------------------main code ------------------------------------

 ;-----------------investigate cme velocity distribution----------------------

   sz        = size(cmemap)
   nimages   = sz(2)
   nangles=sz(1)
   ncmes=max(cmemap)

; determine graphics device
  set_plot, 'Z'
  device, set_resolution = [500,500]   ; size of image
  device, z_buffering    = 0           ; 0 for 2D

for i=1,ncmes do begin
  thiscme=cmecat[i]
  pos=where((cmemap eq i) and (myvelocity gt 0), npositions)   ;pos = positions in the cmemap
  time=pos/nangles
  angle=pos-time*nangles
  cmevelocities=myvelocity[pos]
  med=median(cmevelocities)
  plot,[0,360],[0,0],xrange=[0,360],xstyle=1,yrange=[0,2500],background=255,color=0,charsize=1,linestyle=2,$
  title='Velocity distribution',xtitle='Angle from north',ytitle='Velocity (km/s)',$
  xticks=4,xtickv=[0,90,180,270,360],xminor=8
; xyouts, (angle*2+144) mod 360,cmevelocities,'*',color=0,charsize=2
  plots, (angle*2+144) mod 360,cmevelocities,psym=2,color=0

; -------------------draw boxplot-------------------------------------------
  med=median(cmevelocities)
  groupA=where(cmevelocities le med)
  groupB=where(cmevelocities ge med)
  Q1=median(cmevelocities[groupA])
  Q3=median(cmevelocities[groupB])
  interq=Q3-Q1
  rest=where((cmevelocities ge Q1-interq*1.5) and cmevelocities le Q3+1.5*interq,nr)
  M=minmax(cmevelocities[rest])
  plots, [100,100],[Q1,Q3],color=0  
  plots, [130,130],[Q1,Q3],color=0
  plots, [100,130],[Q1,Q1],color=0
  plots, [100,130],[med,med],color=0
  plots, [100,130],[Q3,Q3],color=0  
  plots,[115,115],[M[0],Q1],color=0
  plots,[115,115],[M[1],Q3],color=0
  plots,[110,120],[M[0],M[0]],color=0
  plots,[110,120],[M[1],M[1]],color=0

; -------------------save plots ----------------------------------------------

  cactus_cme_ID, cmecat[i], ID, yyyy,mm,dd,ho,mi,se,thisdir		;cactus function which extracts the ID, year, month, day, etc ...
  cmedir=dir.out+thisdir						;cactus function

  write_png,cmedir+'speed.png',tvrd()
endfor

END
