;======================== header ===============================================
;+
; NAME:
;   cor2_image_select
; PURPOSE:
;   FUNCTION that selects the good COR2 files for a given list
;
; CATEGORY:
;   cactus ver 4.00 (paperA&A-version, May 2004)
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; INPUTS:
;    list of COR2-images (e.g. output of scclister)   
; 
; KEYWORD PARAMETERS:
;    badfiles ; list of bad images
; OUTPUTS:
;    returns a list of good filenames
;    count ; number of good filenames
;        
; COMMON BLOCKS: 
;   none
; SIDE EFFECTS: 
;   unknown
; RESTRICTIONS:
;   unknown
; PROCEDURE:
;
; CALLS:
;   
;
; TO DO:
;
; MODIFICATION HISTORY:
;   2008/04/07, ER, Created
;   2008/05/15, JMK, added count output parameter [ngood]
;   2008/05/16, ER,  bad is the complement of good in the 'where' search
;   2009/07/16, ER, totally rewritten. Introduced several tests for hdr values
;   2009/08/27, ER, replace keyword 'plot' with 'imname' and do the last statistics on datap98 instead of datap99
;   2010/01/25, ER, changed test for images from before 2007-05-24: they have exptime=2 (instead of 6)
;-
;======================== end header ===========================================
FUNCTION cor2_image_select, filelist, badlist=badlist, imname=imname, ngood, nbad

;   detector='cor2'
   set_plot, 'z'

    ims=sccreadfits(filelist,hdrs,/nodata)
 
 ;check headers for anomalous values:
 
sc=hdrs[0].obsrvtry
IF hdrs[0].obsrvtry eq 'STEREO_A' THEN BEGIN
min_exptime=[5.5,5.5,5.5,5.5] & max_exptime=[6.5,6.5,6.5,6.5]	;The exposure time is very stable, default value is 6
max_datazer=50				; number of zero pixels, it is different from LASCO, default value is 0
min_dataavg=[1160,1160,1160,2540] & max_dataavg=[1250,1250,2600,2600]	
					;dataavg oscillates smoothly as the sc rotates around the sun
					;the average value changed after 2007-10-26T00:07:30.006
					;first value is for before: 2007-10-10T23:37:30.008, second is for after 2007-10-26T00:07:30.006

obs_id_bad=[1785]			;these are "special sequences" that are not useful for cme detection (trigger false alerts)
ENDIF ELSE IF hdrs[0].obsrvtry eq 'STEREO_B' THEN BEGIN
;values to be updated
min_exptime=[1.5,5.5,5.5,5.5] & max_exptime=[2.5,6.5,6.5,6.5]
max_datazer=50
min_dataavg=[1000,1680,1000,2850] & max_dataavg=[1200,1760,3050,3050]
;
obs_id_bad=[1806,1809,1812,1815,1819,1822,1825,1828,1831,1834,1837] ;these all belong to the same campaign on April 22 2009, also in March 9 2009
ENDIF

;initialize
ngood1=0 & ngood2=0 & ngood3=0
nbad1=0 & nbad2=0 & nbad3=0
badlist=' '

;remove files from a "special campaign"
 list=uintarr(n_elements(filelist))
 for i=0,n_elements(filelist)-1 do begin
  z=where(obs_id_bad eq hdrs[i].obs_id,nz)
  list[i]=[nz EQ 0]  ;this is 0 or gt 0, only the files with a 0 have to be retained
 endfor

good1=where(list, ngood1,complement=bad1,ncomplement=nbad1)
if nbad1  gt 0 then badlist =[badlist,filelist[bad1]]
if ngood1 gt 0 then begin 
	filelist=filelist[good1] & hdrs=hdrs[good1]
endif else begin 
	filelist=' '
	goto, EMPTY_LIST
endelse


;check files for atypical header-values: exptime and datazer and dataavg (splitted in two groups)
T0=anytim2tai('2007-05-24T00:08:20.334') & T1=anytim2tai('2007-10-10T23:37:30.008') & T2=anytim2tai('2007-10-26T00:07:30.006')
p0=where(anytim2tai(hdrs.date_obs) le T0, np0)
p1=where((anytim2tai(hdrs.date_obs) gt T0) and (anytim2tai(hdrs.date_obs) le T1), np1)
p2=where((anytim2tai(hdrs.date_obs) gt T1) and (anytim2tai(hdrs.date_obs) lt T2), np2)
p3=where(anytim2tai(hdrs.date_obs) gt T2, np3)

  good2=[1]
  bad2=[1]

if np0 gt 0 then begin ; before 2007-05-24T00:20
  k=0
  hdr=hdrs[p0]
  
  good=where((hdr.exptime GT min_exptime[k])  and (hdr.exptime LT max_exptime[k]) and $
           (hdr.datazer LT max_datazer)  and $
           (hdr.dataavg GT min_dataavg[k])  and (hdr.dataavg LT max_dataavg[k]), $
                  ngood,complement=bad,ncomplement=nbad)
        if ngood gt 0 then good2=[good2,good]
	ngood2=ngood2+ngood
	if nbad gt 0 then bad2=[bad2,bad]
	nbad2=nbad2+nbad
endif

if np1 gt 0 then begin ; before 2007-10-10T23:37
  k=1
  hdr=hdrs[p1]
  

  good=where((hdr.exptime GT min_exptime[k])  and (hdr.exptime LT max_exptime[k]) and $
           (hdr.datazer LT max_datazer)  and $
           (hdr.dataavg GT min_dataavg[k])  and (hdr.dataavg LT max_dataavg[k]), $
                  ngood,complement=bad,ncomplement=nbad)
        if ngood gt 0 then good2=[good2,good]
	ngood2=ngood2+ngood
	if nbad gt 0 then bad2=[bad2,bad]
	nbad2=nbad2+nbad
endif
if np2 gt 0 then begin ; between 2007-10-10T23:37 and 2007-10-26T00:07
  k=2
  hdr=hdrs[p2]
  good=where((hdr.exptime GT min_exptime[k])  and (hdr.exptime LT max_exptime[k]) and $
           (hdr.datazer LT max_datazer)  and $
           (hdr.dataavg GT min_dataavg[k])  and (hdr.dataavg LT max_dataavg[k]), $
                  ngood,complement=bad,ncomplement=nbad)
        if ngood gt 0 then good2=[good2,good]
	ngood2=ngood2+ngood
	if nbad gt 0 then bad2=[bad2,bad]
	nbad2=nbad2+nbad
endif
if np3 gt 0 then begin ; after 2007-10-26T00:07
  k=3
  hdr=hdrs[p3]
  good=where((hdr.exptime GT min_exptime[k])  and (hdr.exptime LT max_exptime[k]) and $
           (hdr.datazer LT max_datazer)  and $
           (hdr.dataavg GT min_dataavg[k])  and (hdr.dataavg LT max_dataavg[k]), $
                  ngood,complement=bad,ncomplement=nbad)
        if ngood gt 0 then good2=[good2,good]
	ngood2=ngood2+ngood
	if nbad gt 0 then bad2=[bad2,bad]
	nbad2=nbad2+nbad
endif

if ngood2 gt 0 then good2=good2[1:*]
if nbad2 gt 0 then bad2=bad2[1:*]

if nbad2  gt 0 then badlist =[badlist,filelist[bad2]]
if ngood2 gt 0 then begin
	filelist=filelist[good2] & hdrs=hdrs[good2]
endif else begin
         filelist=' '
         goto, EMPTY_LIST
endelse


 ;do some extra statistics on the remaining files'
    bkgd=hdrs.datap98/hdrs.exptime
    n=n_elements(bkgd)
    x=findgen(n)
    line=linfit(x,bkgd)
    fit=line[0]+line[1]*x
    sigma=sqrt(total((bkgd-fit)^2)/n)
    rest=where(abs(bkgd - fit) gt 5*sigma,npoints)
    while npoints gt 0 do begin
        bkgd[rest]=fit[rest]
	line=linfit(x,bkgd)
	fit=line[0]+line[1]*x
	sigma=sqrt(total((bkgd-fit)^2)/n)
	rest=where(abs(bkgd - fit) gt 10*sigma,npoints)
    endwhile
    fit=smooth(bkgd,10,/edge_truncate)

    good3=where(abs(hdrs.datap98/hdrs.exptime - fit) lt 3*sigma, ngood3, complement=bad3, ncomplement=nbad3) 
     if nbad3  gt 0 then badlist =[badlist,filelist[bad3]]
     if ngood3 gt 0 then filelist=filelist[good3] else filelist = ' '

if keyword_set(imname) then begin
  plot, hdrs.datap98/hdrs.exptime, yrange=[median(fit)-35*sigma,median(fit)+35*sigma] 
  oplot, fit, color=250,thick=3
  oplot, fit+10*sigma, color=250,linestyle=3
  oplot, fit-10*sigma, color=250,linestyle=3
  if nbad3 gt 0 then plots, x[bad3],fit[bad3],psym=5,symsize=4
  write_png, imname, tvrd()
endif

EMPTY_LIST:

nbad=nbad1+nbad2+nbad3
ngood=ngood3
if nbad gt 0 then badlist=badlist[1:*]


return, filelist

END 
