;+
; NAME: filter_time.pro
; DIR : /fsdata/home/krijger/Idl/Cactus_svn/trunk/idl/
; PURPOSE:
;  
;  
; SAMPLE CALLING SEQUENCE:
;   result=filter_time(data,width=width,sigma=sigma,verbose=verbose)
; INPUT:
;   data - 1D Array
;   width - number of time pixels to convolve over [default 5]
;   sigma - number of standard deviations a bad pixel has to be deviating from local average for replacement [default 3]
;
; OUTPUT:
;   filtered datacurve, all datapoints more then sigma*sdev from time-neighborhood replaced with convolved avg.
;    
; REQUIREMENTS:
;   
; HISTORY:
;   Apr 30 2008 -- 09:58 by JMK - Created
;   2009/08/24, ER, set default width to 5 and default sigma to 3 (as is written in comment above)
;    
;=======================================================================================================
FUNCTION  filter_time,data,$
    	  width=width,quiet=quiet,stop=stop,sigma=sigma,verbose=verbose

if not keyword_set(width) then width=5
if not keyword_set(sigma) then sigma=3

mdata=data

for iwidth=width,3,-2 do mdata=median(mdata,iwidth)

dummy=moment(mdata-data,sdev=sdev_mdata,/nan)
;sdev_mdata=stdev(mdata-data)
good_indices=where(abs(mdata-data) le sigma*sdev_mdata,count)

if count gt 0 then mdata[good_indices]=data[good_indices]
if keyword_set(verbose) then print,'Replacing ',total(finite(data))-count,' pixels'

return,mdata

end
