;-
; PROJECT: CACTUS
;
; PURPOSE: DETECT PLANET APPEARANCES IN COR2 DATA
;
; INPUT: target='mercury' or 'venus'
;
; KEYWORDS: 
;  set /NOBRIGHT if you do not want to test the saturation of the planet (planet mercury may appear as small as a star)
;			;for CACTus only planets that saturate the instrument should be removed
;
; OUTPUT: 
;  result=1 if planet is "visibly" in the FOV
;  pos contains the coordinates of the target (d,lat,long) w.r.t. spacecraft
;
; HISTORY:
;  2009/09/01, ER, Created
;  2009/09/02, ER, Test if coordinates are in the range of the image
;  2009/09/02, ER, Totally rewritten. Only one planet at the time. No wcs needed.
;  2009/09/09, ER, demand that angle is lt hdr.cdelt1*hdr.naxis1/3600/2.
;=======================================================================================
FUNCTION find_planet, hdr,target=target,pos=pos, nobright=nobright

planet=0
fov=0 & bright=1

pos=get_stereo_lonlat(hdr.date_obs, hdr.obsrvtry,  system='hpc',target=target,/degrees)
angle=sqrt((pos[1])^2+(pos[2])^2)

IF (angle le hdr.cdelt1*hdr.naxis1/3600/2.) AND (angle gt 0.5) THEN fov=1		;check if planet if in the fov and not behind occulter
if NOT keyword_set(nobright) then $ 
	IF target eq 'mercury' THEN $ 
		IF (pos[0] lt 1.4e8) THEN bright=0 		;venus is always visible in COR2, mercury only when behind the Sun

planet=fov*bright

return, planet

END
