;======================== header ===============================================
;+
; NAME:
;  JUL_STARTEND
; PURPOSE:
;   This function returns an array with the julian dates of the first and last day of the sequence. 
;   If startdate and enddate are not given it runs in Real-time mode
; CATEGORY:
;   CACTUS, not cactus specific 
;
; INPUTS:
;   [,startdate] [,enddate]  
; KEYWORD PARAMETERS:
;   ndays=ndays  ;default is 4 days for real-time and 1 day if only startdate is given
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)
; OUTPUTS:
;
; COMMON BLOCKS: 
;
; CALLS:
;   
; MODIFICATION HISTORY:
;   2007/12/21, ER, created
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, cactus specific routines (cactus_preamble, cactus_loginfo) are removed 
;   2008/01/09, ER, rename cactus_startend to jul_startend since it is not a cactus-specific routine
;   2008/01/09, ER, move loginfo to cactus_getdata.pro
;   2009/09/03, ER, make juldays of type LONG
;-
;======================== end header ===========================================
FUNCTION jul_startend, ndays=ndays, startdate, enddate

  IF NOT KEYWORD_SET(ndays) THEN $
      IF n_params() EQ 0 THEN ndays=4  $   	; default for real-time
      ELSE IF n_params() EQ 1 THEN ndays=1	; default if startdate is given is 1 

  IF n_params() ne 0 THEN BEGIN			; at least startdate is given
   julstart=str2jul(startdate)
   IF n_params() eq 1 THEN julend=julstart+ndays-1 ELSE julend=str2jul(enddate)
  ENDIF ELSE BEGIN 				; real-time
   today=bin_date(systime(/UTC))
   julend=julday(today[1],today[2],today[0],today[3], today[4])
   julstart=julend-ndays+1
  ENDELSE

 juldays=[julstart,julend]
 return, long(juldays)

END
