;-
; PROJECT: CACTUS
;
; PURPOSE: REMOVE PLANET APPEARANCES IN COR2 DATA
;
; HISTORY:
;  2009/09/01, ER, Created
;  2009/09/03, ER, do one iteration to find the background
;  2009/09/09, ER, Introduce a check on the pixel values of the planet
;  2010/01/25, ER, check if np gt 0 (it may be 0)
;=================================================
FUNCTION remove_planet, im,hdr,target=target,silent=silent,nobright=nobright

planet=find_planet(hdr,target=target,pos=pos,nobright=nobright)
IF NOT planet THEN GOTO, SKIP_REMOVE

wcs=fitshead2wcs(hdr)
pix=wcs_get_pixel(wcs,pos[1:2]*3600) ;*3600 because hdr is in arcsec
;pix contains the pixel-coordinates of the planet
IF NOT PRODUCT(pix lt 1024)*PRODUCT(pix gt 0) THEN GOTO, SKIP_REMOVE  ; if this product is 0 the planet is outsize the FOV

if not keyword_set(silent) then print, 'Planet ',target,' will be removed from image ',hdr.filename
  sz=size(im)
  boxsize=round(1200/hdr.cdelt1)  ;boxsize defined as 600 arcseconds
  box_med=3 > round(270/hdr.cdelt1)
  box_dil=3 > round(150/hdr.cdelt1)
  box_close=1 > round(90/hdr.cdelt1)   
  imlarge=fltarr(sz[1]+2*boxsize,sz[2]+2*boxsize)
  imlarge[boxsize:boxsize+sz[1]-1,boxsize:boxsize+sz[2]-1]=im

  x=pix[0]+boxsize
  y=pix[1]+boxsize
  box=imlarge[x-boxsize:x+boxsize,y-boxsize:y+boxsize]
  
  med=median(box,box_med)
  diff=box-med
  sigma=sqrt(med>0)
  p=where(diff gt sigma,np) 	;do one iteration to reduce the influence of the planet brightness to sigma
  box2=box
  if np gt 0 then box2[p]=!values.f_nan
  med2=median(box2,box_med)
  diff=box-med2
  sigma=sqrt(med2>0)
  
  hits    = morph_close(dilate((diff GT 0.3*sigma),replicate(1,box_dil,box_dil)),replicate(1,box_close,box_close))
	;for illustration set STOP at this point and do:
	;IDL> loadct, 12
	;IDL> !p.multi=[1,2,1]
	;IDL> plot_image, box
	;IDL> plot_image, hits*box
  z=where(hits,nz)
  if nz gt 0 then box(z)=-1

  imlarge[x-boxsize:x+boxsize,y-boxsize:y+boxsize]=box
  im=imlarge[boxsize:boxsize+sz[1]-1,boxsize:boxsize+sz[2]-1]


SKIP_REMOVE:
return, im


END
