;======================== header ===============================================
;+
; NAME:
;   rundiff
; PURPOSE:
;   makes a running ratio difference image of the 2 given images
;
; CATEGORY:
;   cactus
;
; INPUTS:
;   2 images
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;  image with same dimensions as original images
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   
; MODIFICATION HISTORY:
;   2008/03/18, ER, Created, extracted from cactus_reldef
;   2009/04/14, ER, put in keyword ratio if you want running difference ratio images
;-
;======================== end header ===========================================
FUNCTION rundiff, im1, im2,ratio=ratio

; check if images have same dimensions
 sz1=size(im1) & sz2=size(im2)
 npixel1=sz1[1]
 npixel2=sz1[2]

 IF (npixel1 ne sz2[1]) OR (npixel2 ne sz2[2]) THEN print, '! ERROR ! : The two images must have same dimensions!' ELSE BEGIN
  im1_corr=float(im1)
  im2_corr=float(im2)
  mblock=where( (im1 le 0) OR (im2 le 0), nmblock)
  IF nmblock GT 0 THEN BEGIN
    ; avoid edge effects
     dummy1=fltarr(npixel1+10, npixel2+10)
     dummy2=fltarr(npixel1, npixel2)
     dummy1[*]=0
     dummy2[*]=0
     dummy2[mblock]=1
     dummy1[5:(npixel1+4),5:(npixel2+4)]=dummy2
     dummy1=dilate(dummy1, replicate(1,3,3))
     dummy2=dummy1[5:(npixel1+4),5:(npixel2+4)]
     mblock=where(dummy2)
    
     im1_corr[mblock]  = 1
     im2_corr[mblock] = 1
  ENDIF

  IF KEYWORD_SET(ratio) THEN diffim=( im2_corr/im1_corr -1 ) >0 ELSE diffim=im2_corr-im1_corr
  return, diffim
 ENDELSE
END
