;======================== header ===============================================
;+
; NAME:
;   rundiff_cube
; PURPOSE:
;   returns the running difference of a given datcube, and structure index (index.obstime has to contain times of images)
;
; CATEGORY:
;   cactus
;
; INPUTS:
;  datacube[*,*,n] where n is the number of images
;  index: a structure containing info, MUST contain index.obstime (jul-times of observations) 
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;  running difference cube of datacube[*,*,n]
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   rundiff
;   
; MODIFICATION HISTORY:
;   2008/03/26, ER, extracted from cactus_reldev
;   2009/04/14, ER, put in keyword ratio if you want running difference ratio images
;   2009/04/14, ER, move retide=2.5*(retide<0.02) to cactus_reldev (because it is only applied to polar)
;   2009/06/01, ER, make running difference black if roll angle is different between 2 subsequent images (due to SC roll)
;-
;======================== end header ===========================================
FUNCTION rundiff_cube, index, datacube,ratio=ratio

roll=cactus_get_sc_roll(index)

     retide=datacube ; to get the right dimensions
     retide[*]=0
     for i=1, (size(datacube))[3]-1 do begin
       retide[*,*,i]=rundiff(datacube[*,*,i-1],datacube[*,*,i],ratio=ratio)
       dt=2.5* (index[i].obstime-index[i-1].obstime)*24.  ; scale difference with difference time
       retide[*,*,i]=retide[*,*,i]/dt
       if abs(roll[i]-roll[i-1]) gt 0.1 then begin
		retide[*,*,i]=retide[*,*,i]*0
		print, '  '+index[i].filename, ': Made image dark: different rotation angle as previous image'
       endif
     endfor 

return, retide

END
